
//-----------------Field-----------------
//A variable place holder field within an InDesign TextFrame

//param: fieldId is internal InDesign field id of selectable object
EFI.Field = function (fieldId) {
	this.fieldId = fieldId;
	this.fieldNumber = undefined; //field number within Mult Field
	this.originalX = undefined; //used to temporarily store InDesign X
	this.originalY = undefined; //used to temporarily store InDesign Y
	this.x = undefined;
	this.y = undefined;
	this.placeHolder = undefined;
	this.relativeField = undefined;
	this.relativeDirection = undefined;
	this.separatorCharacter = undefined;
	this.separatorSize = undefined;
	this.testData = undefined;
	this.textBlockLine = undefined;
	this.prevField = undefined;
	this.font = undefined;
	this.fontSize = undefined;
	this.colorC = undefined;
	this.colorM = undefined;
	this.colorY = undefined;
	this.colorK = undefined;
	this.alignment = undefined;
	this.rotation = undefined;
	this.relativeToCaption = undefined;
	this.verticalAlign = undefined;
	this.multiline = undefined;
	this.width = undefined;
	this.height = undefined;
	this.defaultText = undefined;
	this.layerId = undefined;

	this.captionDisplayCaption = undefined;
	this.captionDisplayDisplay = undefined;
	this.captionX = undefined;
	this.captionY = undefined;
	this.captionRelativeField = undefined;
	this.captionRelativeDirection = undefined;
	this.captionSeparatorCharacter = undefined;
	this.captionSeparatorSize = undefined;
	this.captionFont = undefined;
	this.captionFontSize = undefined;
	this.captionColorC = undefined;
	this.captionColorM = undefined;
	this.captionColorY = undefined;
	this.captionColorK = undefined;
	this.captionAlignment = undefined;

	this.fieldStartCharacterPosition = undefined;
	this.fieldEndCharacterPosition = undefined;
	this.textObject = undefined;
	this.captionTabbed = false;
	this.fieldTabbed = false;
	this.isAnchorField = false;
};

EFI.Field.prototype.getId = function () {
	return this.fieldId + "-" + this.fieldNumber;
};

EFI.Field.prototype.setFieldNumber = function (fNumber) {
	this.fieldNumber = fNumber;
};

EFI.Field.prototype.setTextBlockLine = function (textBlockLine) {
	this.textBlockLine = textBlockLine;
};

EFI.Field.prototype.setTextObject = function (textObject) {
	this.textObject = textObject;
};

EFI.Field.prototype.getTextObject = function () {
	return this.textObject;
};

EFI.Field.prototype.getTextFrame = function () {
	var textObject = this.getTextObject();
	var textFrame = undefined;

	textFrame = EFI.Utils.getParentTextFrame(textObject);

	return textFrame;
};

EFI.Field.prototype.setFieldCharacterPosition = function () {
	if (this.textObject && this.placeHolder.hasOwnProperty("length")) {
		this.fieldStartCharacterPosition = this.textObject.index;
		this.fieldEndCharacterPosition = this.fieldStartCharacterPosition + this.placeHolder.length;
	}
};

EFI.Field.prototype.getFieldStartCharacterPosition = function () {
	return this.fieldStartCharacterPosition;
};

EFI.Field.prototype.getFieldEndCharacterPosition = function () {
	return this.fieldEndCharacterPosition;
};

EFI.Field.prototype.assignOriginalCoordinates = function (verticalAlign) {
	var textFrame = this.getTextFrame();
	var frameX = EFI.UIHelper.getXCoordinate(textFrame);
	var frameWidth = EFI.UIHelper.getWidth(textFrame);
	var frameY = EFI.UIHelper.getYCoordinate(textFrame);
	var frameHeight = EFI.UIHelper.getHeight(textFrame);

	if (this.multiline) {
		switch (this.alignment) {
			case "center":
				this.x = frameX + (frameWidth / 2);
				break;
			case "right":
				this.x = frameX + frameWidth;
				break;
			case "left":
			default:
				this.x = frameX;
		}
		this.y = frameY;

		this.width = frameWidth;
		this.height = frameHeight;
	} else {
		switch (this.alignment) {
			case "center":
				this.x = frameX + (frameWidth / 2);
				break;
			case "right":
				this.x = frameX + frameWidth;
				break;
			case "left":
			default:
				this.x = this.originalX;
		}

		if (this.isAnchorField && verticalAlign === "center") {
			this.y = frameY + (frameHeight / 2);
		} else {
			this.y = this.originalY;
		}
	}
};

EFI.Field.prototype.getObject = function () {
	//var ruleEngine = new EFI.RuleEngine(this.getTextObject());
	//ruleEngine.processFieldRules();

	var thisObject = {};

	thisObject.type = "text";
	thisObject.id = this.getId();
	thisObject.x = this.x;
	thisObject.y = this.y;
	thisObject.rotation = this.rotation;

	thisObject.placeHolder = this.placeHolder;
	if (this.relativeField) thisObject.relativeFieldId = this.relativeField.getId();
	thisObject.relativeDirection = this.relativeDirection;
	thisObject.separatorCharacter = this.separatorCharacter;
	thisObject.separatorSize = this.separatorSize;
	thisObject.TestData = this.testData;
	thisObject.FontName = this.font;
	thisObject.FontSize = this.fontSize;
	thisObject.CMYKC = this.colorC;
	thisObject.CMYKM = this.colorM;
	thisObject.CMYKY = this.colorY;
	thisObject.CMYKK = this.colorK;
	thisObject.Align = this.alignment;
	thisObject.rotation = this.rotation;
	thisObject.relativeToCaption = this.relativeToCaption;
	thisObject.verticalAlign = this.verticalAlign;
	thisObject.multiline = this.multiline ? "Yes" : "No";

	var visibleCaption = this.placeHolder;
	if (visibleCaption.substring(0, 1) === "[") visibleCaption = visibleCaption.substring(1, visibleCaption.length);
	if (visibleCaption.substring(visibleCaption.length - 1, visibleCaption.length) === "]") visibleCaption = visibleCaption.substring(0, visibleCaption.length - 1);

	if (this.captionDisplayCaption === "") {
		thisObject.captionDisplayCaption = visibleCaption;
	} else {
		thisObject.captionDisplayCaption = this.captionDisplayCaption;
	}

	if (this.captionDisplayCaption) {
		thisObject.captionDisplayDisplay = "Yes";
		thisObject.CaptionDisplayAlternative = visibleCaption;
	} else {
		thisObject.captionDisplayDisplay = "No";
		thisObject.captionDisplayCaption = visibleCaption;
	}

	thisObject.captionX = this.captionX;
	thisObject.captionY = this.captionY;
	if (this.captionRelativeField) thisObject.captionRelativeFieldId = this.captionRelativeField.getId();
	thisObject.captionRelativeDirection = this.captionRelativeDirection;
	thisObject.captionSeparatorCharacter = this.captionSeparatorCharacter;
	thisObject.captionSeparatorSize = this.captionSeparatorSize;
	thisObject.captionFont = this.captionFont;
	thisObject.captionFontSize = this.captionFontSize;
	thisObject.captionColorC = this.captionColorC;
	thisObject.captionColorM = this.captionColorM;
	thisObject.captionColorY = this.captionColorY;
	thisObject.captionColorK = this.captionColorK;
	thisObject.captionAlignment = this.captionAlignment;
	thisObject.defaultWidth = this.width;
	thisObject.defaultHeight = this.height;
	thisObject.defaultText = this.defaultText;
	thisObject.layerId = this.layerId;
	thisObject.fieldId = this.fieldId;

	return thisObject;
};
//-----------------//Field-----------------