﻿
//-----------------ImageField-----------------
//The base structure for storing information about an InDesign ImageFrame

EFI.ImageField = function (imageFrame) {
	EFI.Shape.call(this, imageFrame);
};

//Inherit from the Shape class
EFI.ImageField.inheritsFrom(EFI.Shape);

EFI.ImageField.prototype.hasGraphic = function () {

    var baseObject = this.getBaseObject();
    
	if (baseObject.graphics.length > 0) {
        var graphic = baseObject.graphics[0];
        var itemLink = graphic.itemLink;
		if (itemLink && (itemLink.status === LinkStatus.NORMAL || itemLink.status === LinkStatus.LINK_EMBEDDED)) {
			return true;
		} else {
			return false;
		}

	} else {
		return false;
	}
};

EFI.ImageField.prototype.getGraphic = function () {
	var graphic;

	if (this.hasGraphic()) graphic = this.getBaseObject().graphics[0];

	return graphic;
};

EFI.ImageField.prototype.getImagePath = function () {
	var graphicPath = "";

	if (this.hasGraphic()) graphicPath = this.getFileReference().fullName;
	EFI.Utils.addDebugLog("EFI.ImageField.prototype.getImagePath - " + graphicPath);

	return graphicPath;
};

EFI.ImageField.prototype.getCaption = function () {
	var graphicFilename = "";

	if (this.hasGraphic()) graphicFilename = this.getFileName();
	EFI.Utils.addDebugLog("EFI.ImageField.prototype.getCaption - " + graphicFilename);

	return graphicFilename;
};

EFI.ImageField.prototype.getFileName = function () {
	var graphicFilename = "";

	if (this.hasGraphic()) {
		graphicFilename = this.getFileReference().name;
		aryFilename = graphicFilename.split(":");
		graphicFilename = aryFilename[aryFilename.length - 1];
	}
	EFI.Utils.addDebugLog("EFI.ImageField.prototype.getFileName - " + graphicFilename);

	return graphicFilename;
};

EFI.ImageField.prototype.getInternalGraphicX = function () {
	var x;

	if (this.hasGraphic()) {
		var bounds = this.getGraphic().geometricBounds;
		var boxX = this.getX();

		x = bounds[EFI.ConfigSettings.objectBounds.x1] - boxX;
	}

	return x;
};

EFI.ImageField.prototype.getInternalGraphicY = function () {
	var y;

	if (this.hasGraphic()) {
		var bounds = this.getGraphic().geometricBounds;
		var boxY = this.getY();

		y = bounds[EFI.ConfigSettings.objectBounds.y1] - boxY;
	}

	return y;
};

EFI.ImageField.prototype.getVerticalScale = function () {
	var scale;

	if (this.hasGraphic()) {
        
        var graphic = this.getGraphic();
        
        scale = graphic.verticalScale;
		
        // throws an error if image is a QR Code, even if we check with hasOwnProperty
         try {
             if(graphic.hasOwnProperty ("actualPpi")){
                 //process in terms of 72dpi regardless of actual image DPI
                 scale = scale * (72 / graphic.actualPpi[1]);
             }
         } catch(err) {
             
         }
	}

	return scale;
};

EFI.ImageField.prototype.getInternalImageWidth = function () {
	var width;

	if (this.hasGraphic()) {
		var bounds = this.getGraphic().geometricBounds;

		width = bounds[EFI.ConfigSettings.objectBounds.x2] - bounds[EFI.ConfigSettings.objectBounds.x1];
		width = width / this.getHorizontalScale() * 100;
	}

	return width;
};

EFI.ImageField.prototype.getInternalImageHeight = function () {
	var height;

	if (this.hasGraphic()) {
		var bounds = this.getGraphic().geometricBounds;

		height = bounds[EFI.ConfigSettings.objectBounds.y2] - bounds[EFI.ConfigSettings.objectBounds.y1];
		height = height / this.getVerticalScale() * 100;
	}

	return height;
};

EFI.ImageField.prototype.getInternalScaledImageWidth = function () {
	var width;

	if (this.hasGraphic()) {
		var bounds = this.getGraphic().geometricBounds;

		if (parseFloat(this.getGraphic().rotationAngle) === 90 || parseFloat(this.getGraphic().rotationAngle) === 270) {
			width = bounds[EFI.ConfigSettings.objectBounds.y2] - bounds[EFI.ConfigSettings.objectBounds.y1];
		} else {
			width = bounds[EFI.ConfigSettings.objectBounds.x2] - bounds[EFI.ConfigSettings.objectBounds.x1];
		}
	}

	return width;
};

EFI.ImageField.prototype.getInternalScaledImageHeight = function () {
	var height;

	if (this.hasGraphic()) {
		var bounds = this.getGraphic().geometricBounds;

		if (parseFloat(this.getGraphic().rotationAngle) === 90 || parseFloat(this.getGraphic().rotationAngle) === 270) {
			height = bounds[EFI.ConfigSettings.objectBounds.x2] - bounds[EFI.ConfigSettings.objectBounds.x1];
		} else {
			height = bounds[EFI.ConfigSettings.objectBounds.y2] - bounds[EFI.ConfigSettings.objectBounds.y1];
		}
	}

	return height;
};

EFI.ImageField.prototype.getRotation = function () {
	var rotation;

	if (this.hasGraphic()) {
		rotation = this.getGraphic().rotationAngle;
	}

	return rotation;
};

EFI.ImageField.prototype.getHorizontalScale = function () {
	var scale;

	if (this.hasGraphic()) {
         var graphic = this.getGraphic();
		scale = graphic.horizontalScale;
		//process in terms of 72dpi regardless of actual image DPI

         // throws an error if image is a QR Code, even if we check with hasOwnProperty
         try {
             if(graphic.hasOwnProperty ("actualPpi")){
                scale = scale * (72 / graphic.actualPpi[0]);
             }
         } catch(err) {
             
         }
	}

	return scale;
};

EFI.ImageField.prototype.getFileReference = function () {
	var graphicFile;

	if (this.hasGraphic()) graphicFile = File(this.getGraphic().itemLink.filePath)
	EFI.Utils.addDebugLog("EFI.ImageField.prototype.getFileReference - " + graphicFile);

	return graphicFile;
};

EFI.ImageField.prototype.copyTo = function (locationURI) {
	var copied = false;

	if (this.hasGraphic()) {
		var theGraphic = this.getGraphic();
		EFI.Utils.createCheckFolder(Folder(locationURI));
		var saveTo = locationURI + this.getFileName();         
		EFI.Utils.addDebugLog("EFI.ImageField.prototype.copyTo - " + saveTo);

        var targetPath = new File(saveTo);
        var imageType = theGraphic.imageTypeName.toLowerCase();
        
        // export embedded images
		if (theGraphic.itemLink.status === LinkStatus.LINK_EMBEDDED) {
            
            // we have remove rotation before we export, otherwise the image file gets exported with a rotation
            var oldRotation = this.getRotation();
            this.getGraphic().rotationAngle = 0;
                
            try{
                
                 if(imageType === "jpeg" || imageType === "jpg"){
                     theGraphic.exportFile(ExportFormat.JPG, targetPath);
                     copied = true;
                 } else if(imageType === "png"){
                     theGraphic.exportFile(ExportFormat.PNG_FORMAT, targetPath);
                     copied = true;
                 } else {
                        alert(EFI.I18n.imageField.copyError, EFI.I18n.alertErrorTitle, true);
                 }
             
            } catch(error){
                    copied = false;
            }
        
            // set back original rotation
            this.getGraphic().rotationAngle = oldRotation;
            
		} else if (theGraphic.itemLink.status === LinkStatus.NORMAL) {
			// copy linked images
			copied = this.getFileReference().copy(saveTo);
		}
	}

	return copied;
};

EFI.ImageField.prototype.addAdditionalObjectProperties = function (theObject) {
	theObject.id = this.getId();
	theObject.internalX = this.getInternalGraphicX();
	theObject.internalY = this.getInternalGraphicY();

	theObject.xScale = this.getHorizontalScale();
	theObject.yScale = this.getVerticalScale();
	theObject.imageId = this.getId();
	theObject.captionDisplayCaption = this.getCaption();
	theObject.verticalAlign = "top";
	theObject.Align = "left";

	return theObject;
};

EFI.ImageField.prototype.getImageObject = function (theObject) {
	var imageObject = {};
	var fileName = this.getFileName();

	imageObject.type = "image";
	imageObject.id = this.getId();

	imageObject.placeHolder = this.getCaption();

	imageObject.pathFileName = "Images/";
	imageObject.fileName = imageObject.pathFileName + fileName;

	imageObject.imageWidth = this.getInternalImageWidth();
	imageObject.imageHeight = this.getInternalImageHeight();

	return imageObject;
};

//-----------------//ImageField-----------------