﻿
//-----------------Shape-----------------
//The base structure for storing information about a InDesign object
//This should be inherited from, not used directly

EFI.Shape = function (baseObject) {
	if (baseObject) {
		if (EFI.Utils.hasProperty(baseObject, "id")) this.id = baseObject.id;
		if (EFI.Utils.hasProperty(baseObject, "itemLayer")) this.layerId = baseObject.itemLayer.id;
		this.baseObject = baseObject;
	}
};

EFI.Shape.prototype.getId = function () {
	return this.id;
};

EFI.Shape.prototype.getLayerId = function () {
	return this.layerId;
};

EFI.Shape.prototype.getBaseObject = function () {
	var theBaseObject = undefined;

	if (EFI.Utils.hasProperty(this.baseObject, "id")) theBaseObject = this.baseObject;

	return theBaseObject;
};

EFI.Shape.prototype.getX = function () {
	var bounds = this.getBaseObject().geometricBounds;

	return bounds[EFI.ConfigSettings.objectBounds.x1];
};

EFI.Shape.prototype.getY = function () {
	var bounds = this.getBaseObject().geometricBounds;

	return bounds[EFI.ConfigSettings.objectBounds.y1];
};

EFI.Shape.prototype.getWidth = function () {
	return EFI.UIHelper.getWidth(this.getBaseObject());
};

EFI.Shape.prototype.getHeight = function () {
	var bounds = this.getBaseObject().geometricBounds;
	var height = bounds[EFI.ConfigSettings.objectBounds.y2] - bounds[EFI.ConfigSettings.objectBounds.y1];

	return height;
};

//get populated object that describes the properties of this Shape
EFI.Shape.prototype.getObject = function () {
	var theObject = {};

	theObject.x = this.getX();
	theObject.y = this.getY();
	theObject.width = this.getWidth();
	theObject.height = this.getHeight();

	theObject = this.addAdditionalObjectProperties(theObject);

	return theObject;
};

//Overwrite this function to add shape specific properties
EFI.Shape.prototype.addAdditionalObjectProperties = function (theObject) {
	return theObject;
};

//get user defined properties for the field or place holder field
EFI.Shape.prototype.getProperties = function (placeholder) {
	var selectableObject = this.getSelectableObject();
	var boxProperties = selectableObject.getBoxProperties();
	var fieldProperties;

	if (placeholder) {
		fieldProperties = boxProperties.getUserFieldByPlaceholder(placeholder);
	} else {
		fieldProperties = boxProperties.fieldProperties[0];
	}

	return fieldProperties;
};

//assign the properties to the export object
EFI.Shape.prototype.assignProperties = function (theObject, properties) {
	theObject.properties = properties;
};

EFI.Shape.prototype.getSelectableObject = function () {
	return EFI.ObjectFactory.getInstance("fieldManager").getVariableObject(this.getId());
};

//-----------------//Shape-----------------