﻿
//-----------------TextBlock-----------------
//The base structure for storing information about a InDesign TextFrame

EFI.TextBlock = function (textFrame) {
	this.id = textFrame.id;

	EFI.Utils.addDebugLog("EFI.TextBlock - NEW id: " + this.id);

	this.textFrame = textFrame;
	this.fields; //of Field[]
	this.textBlockLines; //of TextBlockLines
	this.verticalAlign = ""; //top or bottom
	this.initializeFields();
};

EFI.TextBlock.prototype.initializeFields = function () {
	this.fields = [];
	this.textBlockLines = new EFI.TextBlockLines();
};

//param: field
//of type Field
EFI.TextBlock.prototype.addField = function (field, lineNum) {
	EFI.Utils.addDebugLog("EFI.TextBlock.prototype.addField - " + field.placeHolder + " lineNum: " + lineNum);

	if (this.getNumberOfFields() == 0) {
		var selectableObject = EFI.ObjectFactory.getInstance("fieldManager").getVariableObject(this.id);
		if (selectableObject) selectableObject.isVariable = true;
	}

	this.fields.push(field);
	field.setFieldNumber(this.fields.length);
	this.textBlockLines.addField(field, lineNum);
};

EFI.TextBlock.prototype.getFieldByNumber = function (fieldNumber) {
	var field;

	if (fieldNumber <= this.getNumberOfFields()) field = this.fields[fieldNumber - 1];

	return field;
};

EFI.TextBlock.prototype.getFieldsByPlaceholder = function (placeholder) {
	var selectableObject = EFI.ObjectFactory.getInstance("fieldManager").getVariableObject(this.id);
	var aryFields = selectableObject.getUserFieldByPlaceholder(placeholder);

	return aryFields;
};

EFI.TextBlock.prototype.getNumberOfFields = function () {
	return this.fields.length;
};

EFI.TextBlock.prototype.getNumberOfLines = function () {
	return this.textBlockLines.getNumberOfLines();
};

//return copy array of Field objects
EFI.TextBlock.prototype.getFields = function () {
	return this.fields.slice(0);
};

//return all fields and re-setup
EFI.TextBlock.prototype.clearFields = function () {
	delete (this.fields);
	delete (this.textBlockLines);
	this.initializeFields();
};

//return TextBlockLine
EFI.TextBlock.prototype.getLine = function (lineNum) {
	var line = undefined;

	if (lineNum <= this.getNumberOfLines()) line = this.textBlockLines.getLine(lineNum);

	return line;
};

//-----------------//TextBlock-----------------