﻿
//----------------eventMediator-----------
//Does setup and Handling of all Event related activity

$.level = 1;

EFI.EventMediator = function () {
	this.prevSelectedObject;
	this.eventRunning = false; //used for unit testing
};

EFI.EventMediator.prototype.initializeEvents = function() {
	EFI.Utils.addDebugLog("EFI.EventMediator.prototype.initializeEvents");

	var myDocument = EFI.UIHelper.getCurrentDocument();
	myDocument.addEventListener("afterSelectionChanged", this.eventSelectionChanged);
};

EFI.EventMediator.prototype.removeEvents = function() {
	var myDocument = EFI.UIHelper.getCurrentDocument();

	myDocument.removeEventListener("afterSelectionChanged", this.eventSelectionChanged);
};

//------------------------InDesign Canvas Events---------------------------
EFI.EventMediator.prototype.eventSelectionChanged = function(myEvent) {
	//anonymous function so assign "self" as the eventMediator instance
	var self = EFI.ObjectFactory.getInstance("eventMediator");
	self.eventStarted();
	self.objectSelectionChanged();
	self.eventFinished();
};

//param: mockSelectedObject
//dummy selected object for Unit Testing
EFI.EventMediator.prototype.objectSelectionChanged = function(mockSelectedObject) {
	var selectedObject;
	var pluginUI = EFI.ObjectFactory.getInstance("pluginUI");

	pluginUI.setNewFieldState();

	if(mockSelectedObject) {
		selectedObject = mockSelectedObject;
	} else {
		selectedObject = EFI.UIHelper.getSingleSelectedItem();
	}

	EFI.Utils.addDebugLog("EFI.EventMediator.prototype.objectSelectionChanged - " + selectedObject.objectType);

	//check if text in textfield is changed on onBlur
	//this.checkTextUpdate(selectedObject);

	this.prevSelectedObject = selectedObject;

	if(selectedObject.hasSingleSelection) {
		EFI.ObjectFactory.getInstance("FieldManager").setCurrentSelectableObject(selectedObject);

		switch (selectedObject.objectType) {
			case EFI.ConfigSettings.objectTypes.imageframe:
				this.imageFrameSelected(selectedObject);
				//pluginUI.selectTab(EFI.ConfigSettings.menuObjectNames.fields);
				break;
			case EFI.ConfigSettings.objectTypes.textframe:
				this.textFrameSelected(selectedObject);
				//pluginUI.selectTab(EFI.ConfigSettings.menuObjectNames.fields);
				break;
		}
	}
};

//check if text in textfield is changed on onBlur
EFI.EventMediator.prototype.checkTextUpdate = function(selectedObject) {
	EFI.Utils.addDebugLog("EFI.EventMediator.prototype.checkTextUpdate");

	var pluginUI = EFI.ObjectFactory.getInstance("pluginUI");
	var prevSelectedObject = this.prevSelectedObject;

	//if user removed focus from editing text
	if(prevSelectedObject) {
		EFI.Utils.addDebugLog("EFI.EventMediator.prototype.checkTextUpdate - prevSelected: " + prevSelectedObject.objectType + " id " + prevSelectedObject.getId() + " is text type: " + prevSelectedObject.isText() + ". Current " + prevSelectedObject.objectType + " id: " + selectedObject.getId());

		var boxType = prevSelectedObject.getBoxProperties().getBoxType();

		if(prevSelectedObject.isText() && (boxType === "Multi Field" || boxType === "Text Object")) {
			if(!selectedObject.isText() || (prevSelectedObject.getId() !== selectedObject.getId())) {
				if(prevSelectedObject.getBaseObject().parentTextFrames.length > 0) {
					var textField = prevSelectedObject.getBaseObject().parentTextFrames[0];
					EFI.ObjectFactory.getInstance("FieldManager").processTextFieldRules(textField);

					//if(selectedObject) EFI.Utils.addDebugLog("EFI.EventMediator.prototype.checkTextUpdate - id: " + selectedObject.getId() + " fieldsAdded: " + selectedObject.getTextBlock().getNumberOfFields());

					pluginUI.updateFieldList();
				}
			}

		}
	}
};

EFI.EventMediator.prototype.textFrameSelected = function(selectedObject) {
	EFI.Utils.addDebugLog("EFI.EventMediator.prototype.textFrameSelected");

	var pluginUI = EFI.ObjectFactory.getInstance("pluginUI");

	pluginUI.setNewFieldState(selectedObject);
};

EFI.EventMediator.prototype.imageFrameSelected = function(selectedObject) {
	EFI.Utils.addDebugLog("EFI.EventMediator.prototype.imageFrameSelected");
	var pluginUI = EFI.ObjectFactory.getInstance("pluginUI");

	pluginUI.setNewFieldState(selectedObject);
	//pluginUI.selectTab(EFI.ConfigSettings.menuObjectNames.properties);
};

//------------------------End InDesign Canvas Events---------------------------

//-------------------------Field Tree Events-----------

EFI.EventMediator.prototype.initializeTreeEvents = function() {
	//var fieldsTree = EFI.ObjectFactory.getInstance("pluginUI").getMenuObject(EFI.ConfigSettings.menuObjectNames.fieldsTree);
	//if (fieldsTree) fieldsTree.onDoubleClick = this.doubleClickedPluginUIFieldsTree;
};

//When field tree is double clicked
EFI.EventMediator.prototype.doubleClickedPluginUIFieldsTree = function(mockTreeItemSelectionObject) {
	/*var pluginUI = EFI.ObjectFactory.getInstance("pluginUI");
	var fieldManager = EFI.ObjectFactory.getInstance("fieldManager");
	var fieldsTree = pluginUI.getMenuObject(EFI.ConfigSettings.menuObjectNames.fieldsTree);
	var selectedItem;

	if(mockTreeItemSelectionObject) {
		selectedItem = mockTreeItemSelectionObject;
	} else {
		selectedItem = fieldsTree.selection;
	}

	if (selectedItem.type == 'item') {
		var placeHolder = selectedItem.text;
		var selectableObject = fieldManager.getCurrentSelectableObject();

		if(selectableObject) {
			var fieldProperty = selectableObject.boxProperties.getUserFieldByPlaceholder(placeHolder);
			pluginUI.setNewPropertiesState(fieldProperty);

			var field = selectableObject.getTextBlock().getFieldByNumber(selectedItem.efiFieldNumber);
			pluginUI.setupInfoTab(field);

			pluginUI.selectTab(EFI.ConfigSettings.menuObjectNames.properties);
		}
	}*/
};
//----------------------End Field Tree Events-----------

//-------------------Property Tab Save Button---------
//Setup save click for Property tab
EFI.EventMediator.prototype.addOnClickSaveProperty = function(button) {
	EFI.UIHelper.addButtonClickEvent(button, this.onClickSaveProperty);
};

//Setup save click for Fields tab
EFI.EventMediator.prototype.addOnClickSaveFields = function(button) {
	EFI.UIHelper.addButtonClickEvent(button, this.onClickSaveFields);
};

//Setup on change save Fields tab
EFI.EventMediator.prototype.onDropDownChangeSaveFields = function(field) {
	EFI.UIHelper.addDropDownChangeEvent(field, this.onClickSaveFields);
};

//When Property tab save button is clicked
EFI.EventMediator.prototype.onClickSaveProperty = function() {
	EFI.ObjectFactory.getInstance("fieldManager").saveCurrentFieldProperties();
};

//When Fields tab save button is clicked
EFI.EventMediator.prototype.onClickSaveFields = function() {
	EFI.ObjectFactory.getInstance("fieldManager").saveCurrentBoxProperties();
};

//Setup save drop down for Template tab
EFI.EventMediator.prototype.onDropDownChangeSaveTemplate = function(field) {
	EFI.UIHelper.addDropDownChangeEvent(field, this.onClickSaveTemplate);
};

//Setup save drop down for Template tab
EFI.EventMediator.prototype.onTextChangeSaveTemplate = function(field) {
	EFI.UIHelper.addTextChangeEvent(field, this.onClickSaveTemplate);
};

//Setup save click for Template tab
EFI.EventMediator.prototype.addOnClickSaveTemplate = function(button) {
	EFI.UIHelper.addButtonClickEvent(button, this.onClickSaveTemplate);
};

//When Template tab save button is clicked
EFI.EventMediator.prototype.onClickSaveTemplate = function() {
	EFI.ObjectFactory.getInstance("fieldManager").saveCurrentTemplateProperties();
};

//Setup save click for Template tab
EFI.EventMediator.prototype.addOnClickExportTemplate = function(button) {
	EFI.UIHelper.addButtonClickEvent(button, this.onClickExportTemplate);
};

//When Template tab export button is clicked
EFI.EventMediator.prototype.onClickExportTemplate = function () {
	var myFile = File.saveDialog(EFI.I18n.fileExport.exportDialogTitle, "*.zip");

	if(myFile) {
		var exportFile = File(myFile.absoluteURI);
		var exporter = new EFI.Exporter(exportFile.fullName);

		exporter.exportPackage();
		alert(EFI.I18n.fileExport.exportComplete, EFI.I18n.fileExport.exportCompleteDialogTitle);
	}
};

//Setup clear state click
EFI.EventMediator.prototype.addOnClickClearState = function(button) {
	EFI.UIHelper.addButtonClickEvent(button, this.onClickClearState);

};

//Clear save state of document
EFI.EventMediator.prototype.onClickClearState = function() {
	var fieldManager = EFI.ObjectFactory.getInstance("fieldManager");
	fieldManager.clearAll();
	fieldManager.saveState();
};

//Setup check text click
EFI.EventMediator.prototype.addOnClickCheckText = function(button) {
	EFI.UIHelper.addButtonClickEvent(button, this.onClickSaveFields);

};

//Check text
EFI.EventMediator.prototype.onClickCheckText = function() {
	var textFrame = EFI.TestHelper.getTextFrameObject("Multi line");

	var xmlEngine = new EFI.XMLEngine(textFrame);
	var strXML = xmlEngine.generateXML();

	EFI.Utils.addDebugLog(strXML);
	alert(strXML);
};

//---------------------END Property Tab Save Button---------
EFI.EventMediator.prototype.eventFinished = function() {
	this.eventRunning = false;
};

EFI.EventMediator.prototype.eventStarted = function() {
	this.eventRunning = true;
};

EFI.EventMediator.prototype.hasEventFinished = function() {
	return !this.eventRunning;
};

EFI.EventMediator.prototype.clearState = function() {
	this.prevSelectedObject = undefined;
	this.eventRunning = false;
};
//----------------//eventMediator-----------