
//----------------Exporter-------------
//Generates Export package

EFI.Exporter = function(exportFileURI, _onProgressUpdate) {
    //EFI.Utils.debug(EFI.UIHelper.getCurrentDocument().extractLabel("EFISmartCanvasExporter"));
    this.exportFileURI = exportFileURI;
    this.template = {};
    this.layers = new EFI.Layers();
    this.variableImageShape = [];
    this.docWidth=EFI.Utils.getDocumentWidth();
    this.docHeight=EFI.Utils.getDocumentHeight();
    this.PageNumber = 1;
    this.onProgressUpdate = function(message) {
        if(typeof onProgressUpdate === "function"){
            _onProgressUpdate(message);
        } 
    } 
    
    // while exporting, we collect the IDs of the variable objects we exported
    // so we can avoid exporting the same object multiple times
    this.exportedVariableObjectIds = [];
};

EFI.Exporter.prototype.getTempFolder = function() {
    this.tempFolder = EFI.Utils.createCheckFolder(new Folder(EFI.Utils.getTempFolder() + "/randomFolder"));
    
    return this.tempFolder;
}


EFI.Exporter.prototype.exportPackage = function(progressBar) {
    var result = {
        success: true,
        errorMessage: ""
    };

    try {
        this.exportedVariableObjectIds = [];
        
        var curDoc = EFI.UIHelper.getCurrentDocument();
        
        this.measurementUnits = curDoc.viewPreferences.horizontalMeasurementUnits;
            
        var xOrig = EFI.UIHelper.switchXToPoints();
        var yOrig = EFI.UIHelper.switchYToPoints();

        
        var zeroPoint = curDoc.zeroPoint;
        curDoc.zeroPoint = [0,0];
        
        this.onProgressUpdate("Cleaning temp files");
        this.cleanTempFiles();
        
        this.onProgressUpdate("Exporting layers");
        this.exportLayerFiles();
        if(!this.exportJSON()){            
                result.success = false;
                result.errorMessage = "Error: Could not write DocMl file!";
        }
        this.getFonts();
        //this.exportImages();

        this.onProgressUpdate("Creating ZIP package");
        EFI.Utils.zipFolder(File(this.getTempFolder()), File(this.exportFileURI));
        
        this.cleanTempFiles();

        curDoc.zeroPoint = zeroPoint;
        
        EFI.UIHelper.switchXToOriginal(xOrig);
        EFI.UIHelper.switchYToOriginal(yOrig);

        this.onProgressUpdate("Finished export");

    } catch(error) {
        EFI.Utils.addDebugLog(error);

        result.success = false;
        result.errorMessage = error;
    }

    return result;
}

EFI.Exporter.prototype.exportJSON = function() {

    try {
        this.onProgressUpdate("Processing...");
        
        EFI.Utils.addDebugLog("EFI.Exporter.prototype.exportFile - to " + this.exportFileURI );

        this.getTemplateExportObject();
        var theLayers = this.layers.getLayers();

        this.template.fields = [];
        this.template.images = [];

        for(var i = 0; i < theLayers.length; i++) { 
            this.onProgressUpdate("Processing layer " + (i+1) + " of " + theLayers.length);
        
            var layerSegments = theLayers[i].getLayerSegments();
            //alert(layerSegments.length);
            for (var j = 0; j < layerSegments.length; j++) {
                this.onProgressUpdate("Processing layer items " + (j+1) + " of " + layerSegments.length);
        
                var layerSegment = layerSegments[j];
                var exportObjects;

                if (layerSegment.isStatic()) {
                    exportObjects = this.getExportObject(undefined, layerSegment);
                } else {
                    var selectedObjectToExport = layerSegment.getSelectableObject();
                    var boxProperties = selectedObjectToExport.getBoxProperties();
                    var multiline = boxProperties.boxType.fieldValue === "Text Object";

                    if (boxProperties.boxType.fieldValue === "Multi Field") {
                        var baseObject = selectedObjectToExport.getBaseObject();

                        if (baseObject) {
                            var ruleEngine = new EFI.RuleEngine(selectedObjectToExport.getBaseObject(), multiline);
                            ruleEngine.processFieldRules();
                        }
                    }

                    exportObjects = this.getExportObject(selectedObjectToExport);
                }

                this.addExportObjects(exportObjects);
            }
        }

        return this.saveExportFile();

    } catch(error) {
        EFI.Utils.addDebugLog(error);
    }

};

EFI.Exporter.prototype.exportPDF = function(type) {
    EFI.Utils.addDebugLog("EFI.Exporter.prototype.exportPDF - to " + this.exportFileURI );

    var fieldManager = EFI.ObjectFactory.getInstance("fieldManager");
    var fields = fieldManager.getVariableObjects();
    var exportType = "", folder = "", fileName = "", profile = "";
    var hideIt = false;
    var i = 0;
    var selectedObject = undefined;

    switch (type) {
        case "low":
            exportType = "LowRes Merge File Only";
            fileName = "lowres.pdf";
            folder = "TemplateLowResMergeFile";
            profile = "[High Quality Print]";
            break;
        case "high":
            exportType = "HiRes Merge File Only";
            fileName = "highres.pdf";
            folder = "TemplateMergeFile";
            profile = "[High Quality Print]";
            break;
        case "shell":
            exportType = "Shell File Only";
            fileName = "shell.pdf";
            folder = "TemplateShellFile";
            profile = "[High Quality Print]";
            break;
        case "template":
            exportType = "Multi Field";
            fileName = "template.pdf";
            folder = "TemplateTemplatePDFFile";
            profile = "[High Quality Print]";
            break;
    }

    //"HiRes Merge File Only", "LowRes Merge File Only", "Both Merge Files","Do Not Export"
    for(i = 0; i < fields.length; i++) {
        selectedObject = fields[i];
        var boxType = selectedObject.boxProperties.getBoxType();

        hideIt = false;

        if(boxType !== exportType) {
            if ((exportType === "HiRes Merge File Only" || exportType === "LowRes Merge File Only") && boxType !== "Both Merge Files") hideIt = true;
            if (exportType === "Multi Field" && boxType !== "Resource File Only") hideIt = true;
            if (exportType === "Shell File Only") hideIt = true;

            if(hideIt) selectedObject.hide();
        }
    }

    var pdfExporter = new EFI.PDFEngine();
    var fullFolder = this.getTempFolder() + "/" + folder + "/";
    var exportFolder = Folder(fullFolder);
    var exportFile = File(fullFolder + fileName);

    pdfExporter.exportPDF(exportFolder, exportFile, profile);

    for(i = 0; i < fields.length; i++) {
        selectedObject = fields[i];
        selectedObject.display();
    }

    //ToDo: Check if export file exists
    return true;
};

EFI.Exporter.prototype.exportLayerFiles = function() {
    var fieldManager = EFI.ObjectFactory.getInstance("fieldManager");
    var profile = "[High Quality Print]";
    var folder = "Images";
    var fullFolder = this.getTempFolder() + "/" + folder + "/";
    var exportFolder = Folder(fullFolder);

    this.layers.exportStaticPDFs(exportFolder, profile);
};

EFI.Exporter.prototype.getTemplateExportObject = function() {
    var fieldManager = EFI.ObjectFactory.getInstance("fieldManager");

    var templateObject = fieldManager.getCurrentTemplateProperties();

    this.template = EFI.Utils.cleanObjectProperties(templateObject);
};

//return array of all exportable objects for this selectable object
EFI.Exporter.prototype.getExportObject = function(selectedObjectToExport, staticLayer) {
    var exportObjects = {};
    exportObjects.fields = [];
    exportObjects.images = [];

    var boxType;
    if(selectedObjectToExport) {
        boxType = selectedObjectToExport.boxProperties.boxType.fieldValue;
    } else {
        boxType = "Static";
    }

    if(boxType === "Static") {
        exportObjects.fields[0] = staticLayer.getObject();
    } else if(boxType === "Text Object") {
        exportObjects.fields = selectedObjectToExport.getShape().getObjects();
    }else{
        exportObjects.fields[0] = selectedObjectToExport.getShape().getObject();

        switch (selectedObjectToExport.boxProperties.getBoxType()) {
            case "Image Object":
                var shape = selectedObjectToExport.getShape();
                if(shape.hasGraphic()) {
                    var imageObject = this.processImageForExport(selectedObjectToExport);
                    exportObjects.images.push(imageObject);
                }

                break;
            case EFI.ConfigSettings.objectTypes.textframe:
                //this.processImageForExport(selectedObjectToExport);
                break;
        }
    }

    return exportObjects;
};

EFI.Exporter.prototype.processImageForExport = function(selectedObjectToExport) {
    var shape = selectedObjectToExport.getShape();
    
    var baseImage = shape.baseObject.images[0];
    var baseGraphic = shape.baseObject.graphics[0];
    
    var imageObject = shape.getImageObject();
    
    var fileName = shape.getFileName().toLowerCase();

    if(EFI.Utils.hasImageFileExtension(fileName)) {
        shape.copyTo(this.getTempFolder() + "/" + imageObject.pathFileName);
    } 

    this.variableImageShape.push(shape);
    return imageObject;
};

EFI.Exporter.prototype.addExportObjects = function(exportObjects) {
    if(exportObjects.fields.length > 0) this.template.fields = this.template.fields.concat(exportObjects.fields);
    if(exportObjects.images.length > 0) this.template.images = this.template.images.concat(exportObjects.images);
};

EFI.Exporter.prototype.saveExportFile = function() {
    //ADD BACK IN
    var XML = this.getDocML();
    var xmlFile = File(this.getTempFolder() + "/document.xml");
    xmlFile.encoding = "UTF-8";
    
    return EFI.Utils.writeFile(xmlFile, XML);
};

EFI.Exporter.prototype.getDocML = function() {
    var XML = new XMLWriter();

    this.addDocument(XML);
    XML.Close();

    return XML.ToString();
};

EFI.Exporter.prototype.addDocument = function(XML) {
    
    this.onProgressUpdate("Creating SmartCanvas DocMl file...");
    
    var myDocument = EFI.UIHelper.getCurrentDocument();
    
    XML.BeginNode("Document");

    XML.Attrib("ID", EFI.Utils.getGUID());
    XML.Attrib("Version", "1.0");
    XML.Attrib("InsertPageWidth", this.docWidth);
    XML.Attrib("InsertPageHeight", this.docHeight);
    XML.Attrib("Width", this.docWidth);
    XML.Attrib("Height", this.docHeight);

    // BLEED
    var bleedTop = EFI.Utils.getBleedTop();
    var bleedBottom = EFI.Utils.getBleedBottom();
    var bleedLeft = EFI.Utils.getBleedLeft();
    var bleedRight = EFI.Utils.getBleedRight();
    
    var bleedActive = bleedTop || bleedBottom || bleedLeft || bleedRight;
    
    if(bleedActive){
        XML.Attrib("PrintBleedActive", true);
        
        XML.Attrib("PrintBleedTop", bleedTop);
        XML.Attrib("PrintBleedBottom", bleedBottom);
        XML.Attrib("PrintBleedLeft", bleedLeft);
        XML.Attrib("PrintBleedRight", bleedRight);
        
        XML.Attrib("BleedTop", bleedTop);
        XML.Attrib("BleedBottom", bleedBottom);
        XML.Attrib("BleedLeft", bleedLeft);
        XML.Attrib("BleedRight", bleedRight);
    }

    // MARGIN
    var marginTop = myDocument.marginPreferences.top;
    var marginBottom = myDocument.marginPreferences.bottom;
    var marginLeft = myDocument.marginPreferences.left;
    var marginRight = myDocument.marginPreferences.right;
    
    XML.Attrib("MarginTop", marginTop);
    XML.Attrib("MarginBottom", marginBottom);
    XML.Attrib("MarginInside", marginLeft);
    XML.Attrib("MarginOutside", marginRight);
    
    var hasFacingPages = myDocument.documentPreferences.facingPages;
    if(hasFacingPages){
        XML.Attrib("ConvertToHasFacingPages", true);
    }
    var a = app;

	var unit= EFI.Utils.getSmartCanvasLengthUnitFromMeasurementUnit( this.measurementUnits );
	XML.Attrib("UI-DefaultUnit", unit);
    
    var refPoint = app.activeWindow.transformReferencePoint;
    var smartRefPoint = EFI.Utils.getSmartCanvasReferencePoint(refPoint);    
    XML.Attrib("UI-ReferencePoint",  smartRefPoint);

    XML.BeginNode("Resources");
    this.addSwatches(XML);
    this.addTextStyles(XML);
    XML.EndNode(); //Resources

    var strPageName = "";
    var intActivePage = 0;

    strPageName=app.activeWindow.activePage.name;
    for(var i = 0; i < app.activeDocument.pages.length; i++) {

        this.onProgressUpdate("Processing page " + (i+1) + " of " + app.activeDocument.pages.length);
        
        app.activeWindow.activePage = app.activeDocument.pages[i];
        if(strPageName === app.activeWindow.activePage.name) intActivePage = i;

        this.PageNumber = i + 1;
        this.addPage(XML);

    }
    app.activeWindow.activePage = app.activeDocument.pages[intActivePage];
    this.addLayers(XML);

    XML.EndNode(); //Document
    
};

EFI.Exporter.prototype.addPage = function(XML) {
    var pageDocW=0;
    var pageDocH=0;
    var gb = app.activeDocument.pages[this.PageNumber - 1].bounds;

    pageDocW=gb[3]-gb[1];
    pageDocH=gb[2]-gb[0];


    XML.BeginNode("Page");
    XML.Attrib("Width", pageDocW);
    XML.Attrib("Height", pageDocH);

    XML.BeginNode("Frame");
    XML.Attrib("Width", pageDocW);
    XML.Attrib("Height", pageDocH);

    XML.BeginNode("Composition");
    XML.Attrib("Width", pageDocW);
    XML.Attrib("Height", pageDocH);
    XML.Attrib("Zorder", "increasing");

    this.addFields(XML);

    XML.EndNode(); //Composition
    XML.EndNode(); //Frame
    XML.EndNode(); //Page
};

EFI.Exporter.prototype.addLayers = function(XML) {
    var layers = EFI.UIHelper.getLayers();

    if(layers.length > 0) {
        XML.BeginNode("Layers");

        // add the default layer
        XML.BeginNode("Layer");
        XML.Attrib("Name", "Default");
        XML.Attrib("IsDefault", "true");            
        XML.EndNode(); //Layer
        
        // reverse order to show up correctly in SmartCanvas
        for (var i = layers.length - 1; i >= 0; i--) {
            var layer = layers[i];

            XML.BeginNode("Layer");

            XML.Attrib("Name", layer.id);
            XML.Attrib("Color", layer.layerColor.toString().toLowerCase().replace("_", ""));
            XML.Attrib("DisplayName", layer.name);
            XML.Attrib("IsLocked", layer.locked);

            if(!layer.visible){
                XML.Attrib("SwitchMode", "Never");
            }

            XML.EndNode(); //Layer
        }
    
        XML.EndNode(); //Layers
    }
};

EFI.Exporter.prototype.addFields = function (XML) {
    
    for(var i = 0; i < this.template.fields.length; i++) {
        var fieldObject = this.template.fields[i];

        switch (fieldObject.type) {
            case "text":
                var strTFPageNo=999;

                var tf = null;
                
                //GetPageNumber
                var allTextFrames = app.activeDocument.textFrames;
                for (var j = 0; j < allTextFrames.length; j++) {

                    if (allTextFrames[j].id.toString() === fieldObject.fieldId.toString()) {
                        if(allTextFrames[j] && allTextFrames[j].parentPage){
                            strTFPageNo = allTextFrames[j].parentPage.name;
                            tf = allTextFrames[j];
                        }
                    }
                }

                if (strTFPageNo.toString() === this.PageNumber.toString() && tf) {
                    this.addTextField(XML, fieldObject, tf);
                }
                break;
            case "image":
                this.addImageField(XML, fieldObject);
                break;
                
            default:
                this.addVariableImage(XML, fieldObject);
                break;
                
        }

    }
    

};

EFI.Exporter.prototype.addVariableImage = function (XML, fieldObject) {
    
    var document = EFI.UIHelper.getCurrentDocument();
    var pageWidth = document.documentPreferences.pageWidth;
    var hasFacingPages = document.documentPreferences.facingPages;
    var pageNumber = this.PageNumber;
    var pagesPerDocument = document.documentPreferences.pagesPerDocument;
    
    for (var i = 0; i < this.variableImageShape.length; i++) {

        var objID = this.variableImageShape[i].id;

        if(objID !== fieldObject.id){
                continue;
        }
        
        var shapeBounds = EFI.Utils.getFrameCoOrdinates(this.variableImageShape[i]);
        var strTFPageNo = 999;
        

        //GetPageNumber
        var allImageFrames = app.activeDocument.allPageItems;
        var imageFrame = null;
        for (var j = 0; j < allImageFrames.length; j++) {
            
            if (allImageFrames[j].id === objID) {
                imageFrame = allImageFrames[j];
                strTFPageNo = imageFrame.parentPage.name;
                break;
            }
        }

        if (parseInt(this.PageNumber) === parseInt(strTFPageNo)) {
            
            var leftPosition = shapeBounds.Left;
            
            if(hasFacingPages && pageNumber > 2 && !EFI.Utils.isEven(pageNumber)){        
                    leftPosition -= pageWidth;
            }


            XML.BeginNode("Picture");
            XML.Attrib("ID", EFI.Utils.getGUID());
            XML.Attrib("Source", "template/" + decodeURIComponent(shapeBounds.FileName));
//            XML.Attrib("Layer", "999-" + i);
            
            XML.Attrib("Layer", this.variableImageShape[i].layerId);
            XML.Attrib("Page", this.PageNumber);

            XML.Attrib("Left", leftPosition);
            XML.Attrib("Top", shapeBounds.Top);
            XML.Attrib("Width", shapeBounds.Width);
            XML.Attrib("Height", shapeBounds.Height);

            XML.Attrib("Rotation", shapeBounds.Rotation);

            XML.Attrib("PadLeft", shapeBounds.PadLeft);
            XML.Attrib("PadTop", shapeBounds.PadTop);
            XML.Attrib("PadRight", shapeBounds.PadRight);
            XML.Attrib("PadBottom", shapeBounds.PadBottom);
            XML.Attrib("Opacity", shapeBounds.Opacity);
            
            XML.Attrib("Stretch", "UniformToFill");
            XML.Attrib("CustomStretch", "UniformToFill");

            if(imageFrame && imageFrame.fillColor){
                XML.Attrib("Background", EFI.Utils.getSwatchOrColor(imageFrame.fillColor));
            }
        
            if (shapeBounds.BorderColor !== "") XML.Attrib("BorderColor", shapeBounds.BorderColor);
            XML.Attrib("BorderThickness", shapeBounds.BorderThickness);
            XML.Attrib("CornerRadius", shapeBounds.CornerRadius);


        // drop shadow
        if(imageFrame && imageFrame.transparencySettings && imageFrame.transparencySettings.dropShadowSettings && imageFrame.transparencySettings.dropShadowSettings.mode === ShadowMode.DROP){
    
            XML.BeginNode("DropShadow");
            
            XML.Attrib("XOffset", EFI.Utils.roundNumber(imageFrame.transparencySettings.dropShadowSettings.xOffset, 4));
            XML.Attrib("YOffset", EFI.Utils.roundNumber(imageFrame.transparencySettings.dropShadowSettings.yOffset, 4));
            XML.Attrib("Size", EFI.Utils.roundNumber(imageFrame.transparencySettings.dropShadowSettings.size, 4));
            XML.Attrib("Color", EFI.Utils.getSwatchOrColor(imageFrame.transparencySettings.dropShadowSettings.effectColor));
            
            
            var blendMode = imageFrame.transparencySettings.dropShadowSettings.blendMode === BlendMode.MULTIPLY ? "Multiply" : "Normal";
            XML.Attrib("BlendMode", blendMode);
            
            XML.EndNode();
        }

            XML.EndNode(); //Image
        }
    }

};

EFI.Exporter.prototype.addTextField = function(XML, fieldObject, textFrame) {
    
    var document = EFI.UIHelper.getCurrentDocument();
    var pageWidth = document.documentPreferences.pageWidth;
    var hasFacingPages = document.documentPreferences.facingPages;
    var pageNumber = this.PageNumber;
    var pagesPerDocument = document.documentPreferences.pagesPerDocument;
    
    var plainId = EFI.Utils.getPlainId(fieldObject.id);
    
    // first check if we have allready exported this fieldObject
    if(EFI.Utils.arrayContains(this.exportedVariableObjectIds, plainId)){
        return;
    }
    
    this.exportedVariableObjectIds.push(plainId);
    
    
    var originalRotation = textFrame.rotationAngle;
    
    var curRotation = fieldObject.rotation;
   
    if (curRotation !== 0) {
        if (curRotation > 0) {
            curRotation = 360 - curRotation;
        } else {
            curRotation = - 1 * curRotation;
        }
    }

    XML.BeginNode("Text");

    XML.Attrib("ID", EFI.Utils.getPlainId(fieldObject.id));

    XML.Attrib("Rotation", curRotation);

    // we need to set the rotation angle temporary to 0 to get the correct position and size
    textFrame.rotationAngle = 0;
    
    var leftPosition = EFI.Utils.roundNumber(EFI.UIHelper.getXCoordinate(textFrame), 4)
    
    if(hasFacingPages && pageNumber > 2 && !EFI.Utils.isEven(pageNumber)){        
            leftPosition -= pageWidth;
    }
    
    XML.Attrib("Left", leftPosition);
    XML.Attrib("Top", EFI.Utils.roundNumber(EFI.UIHelper.getYCoordinate(textFrame), 4));
    XML.Attrib("Width", EFI.Utils.roundNumber(EFI.UIHelper.getWidth(textFrame), 4));
    XML.Attrib("Height", EFI.Utils.roundNumber(EFI.UIHelper.getHeight(textFrame), 4));

    if(textFrame.fillColor){
        XML.Attrib("Background", EFI.Utils.getSwatchOrColor(textFrame.fillColor));
    }

    var opacity = EFI.Utils.roundNumber(parseFloat(textFrame.transparencySettings.blendingSettings.properties.opacity) / 100, 4);
    XML.Attrib("Opacity", opacity);

    var strokeWeight = textFrame.strokeWeight;
    if(strokeWeight !== 0) {        
        XML.Attrib("BorderColor", EFI.Utils.getSwatchOrColor(textFrame.strokeColor));
        XML.Attrib("BorderThickness", strokeWeight);
    }

    XML.Attrib("VerticalAlignment", EFI.UIHelper.getVerticalAlign(textFrame));
            
    XML.Attrib("Layer", fieldObject.layerId);
    XML.Attrib("Version", "2");
    
    XML.Attrib("FitMode", "ShrinkOnly");

    // always turn on pixel perfect text rendering
    XML.Attrib("HtmlPreview", false);

    // set text flow
    if(textFrame.previousTextFrame && textFrame.previousTextFrame.id) {
            XML.Attrib("FlowPredecessor", EFI.Utils.getPlainId(textFrame.previousTextFrame.id));
    }
    if(textFrame.nextTextFrame && textFrame.nextTextFrame.id) {
            XML.Attrib("FlowSuccessor", EFI.Utils.getPlainId(textFrame.nextTextFrame.id));
    }

    // set the paragraph style
    if(textFrame.paragraphs && textFrame.paragraphs.length > 0){
        var paragraph = textFrame.paragraphs[0];
        if(paragraph && paragraph.appliedParagraphStyle){
            XML.Attrib("BasedOn", paragraph.appliedParagraphStyle.name);
        }
    }

    // drop shadow
    if(textFrame.transparencySettings && textFrame.transparencySettings.dropShadowSettings && textFrame.transparencySettings.dropShadowSettings.mode === ShadowMode.DROP){
    
        XML.BeginNode("DropShadow");
        
        XML.Attrib("XOffset", EFI.Utils.roundNumber(textFrame.transparencySettings.dropShadowSettings.xOffset, 4));
        XML.Attrib("YOffset", EFI.Utils.roundNumber(textFrame.transparencySettings.dropShadowSettings.yOffset, 4));
        XML.Attrib("Size", EFI.Utils.roundNumber(textFrame.transparencySettings.dropShadowSettings.size, 4));
        XML.Attrib("Color", EFI.Utils.getSwatchOrColor(textFrame.transparencySettings.dropShadowSettings.effectColor));
        
        var blendMode = textFrame.transparencySettings.dropShadowSettings.blendMode === BlendMode.MULTIPLY ? "Multiply" : "Normal";
        XML.Attrib("BlendMode", blendMode);
        
        XML.EndNode();
    }
    
    // now set the original angel back;
    textFrame.rotationAngle = originalRotation;
    
    
    // Don't use text of successor text boxes, becuase all text is already in the first predecessor text box
    if(!textFrame.previousTextFrame|| !textFrame.previousTextFrame.id) {

        if(EFI.ConfigSettings.outputType === "InDesign Text") {
            this.addInDesignText(XML, fieldObject);
        } else {
            this.addParagraph(XML, fieldObject);
        }

    } else {
            XML.BeginNode("Paragraph");
            XML.EndNode();
    }

    XML.EndNode(); //Text
};

EFI.Exporter.prototype.addInDesignText = function(XML, fieldObject) {
    var baseTextProperties = EFI.TextFormatHelper.getBaseTextProperties();
    XML.BeginNode("IDTT");
    //XML.Attrib("FontFamily", baseTextProperties.fontName);
    //XML.Attrib("Fontsize", baseTextProperties.pointSize);

    XML.WriteCDATA(fieldObject.defaultText);

    XML.EndNode(); //IDTT
};

EFI.Exporter.prototype.addParagraph = function(XML, fieldObject) {
    var baseTextProperties = EFI.TextFormatHelper.getBaseTextProperties();
    XML.BeginNode("Paragraph");
    XML.Attrib("FontFamily", baseTextProperties.fontName);
    XML.Attrib("Fontsize", baseTextProperties.pointSize);

    XML.BeginNode("Run");
    //XML.Attrib("FontFamily", fieldObject.FontName);
    XML.Attrib("FontFamily", "Arial");
    XML.Attrib("Fontsize", fieldObject.FontSize);
    XML.WriteString(fieldObject.defaultText);

    XML.EndNode(); //Run
    XML.EndNode(); //Paragraph
};

EFI.Exporter.prototype.addImageField = function (XML, fieldObject) {

    if (XML && fieldObject && fieldObject.fileName) {
        var strFileName = fieldObject.fileName.absoluteURI;
        var strRoot = encodeURI(this.getTempFolder());
        //Do Images vs Template patch here.
        var strLayerLink = strFileName.replace(strRoot, "").replace("/Images/", "template/");
        XML.BeginNode("Picture");

        // respect bleed
        var bleedLeft = EFI.Utils.getBleedLeft();
        var bleedTop = EFI.Utils.getBleedTop();
        var bleedRight = EFI.Utils.getBleedRight();
        var bleedBottom = EFI.Utils.getBleedBottom();
        
        var left = fieldObject.x - bleedLeft;
        var top = fieldObject.y - bleedTop;
        var width = fieldObject.width + bleedLeft + bleedRight;
        var height = fieldObject.height + bleedTop + bleedBottom;

        //XML.Attrib("ID", fieldObject.id);
        XML.Attrib("ID", EFI.Utils.getGUID());
        XML.Attrib("Source", strLayerLink);
        XML.Attrib("Layer", fieldObject.layerId);
        XML.Attrib("Left", left);
        XML.Attrib("Top", top);
        XML.Attrib("Width", width);
        XML.Attrib("Height", height);
        XML.Attrib("Page", this.PageNumber);
        XML.Attrib("IsLocked", "true");

        var pageCount = app.activeDocument.pages.length;
        XML.Attrib("PageCount", pageCount);
        
        XML.EndNode(); //Picture
    }
};

EFI.Exporter.prototype.cleanTempFiles = function() {
    EFI.Utils.deleteFolder(Folder(this.getTempFolder()));
};

EFI.Exporter.prototype.addFont = function(fontFileLocation) {
    
    var myFile = File(fontFileLocation);
    var myFileName = myFile.name.substr(0,myFile.name.lastIndexOf("."));
    var myFileExt = myFile.name.substr(myFile.name.lastIndexOf(".") + 1,myFile.name.length);
    var destFolder = this.getTempFolder() + "/fonts";
    EFI.Utils.copyFile(fontFileLocation, EFI.Utils.createCheckFolder(Folder(destFolder)));

};

EFI.Exporter.prototype.getFonts = function() {
                
        var documentFonts = app.activeDocument.fonts;
        var f = documentFonts.everyItem().getElements();
        var fontLocation = "";
        for (var i = 0; i < f.length; i++) {
            var font = f[i];
            fontLocation = "";
            try {
                fontLocation = String (font.location);
            } catch (_) {
                fontLocation = String (font.location);                
            }
            if (fontLocation !== "") {

                if(EFI.Utils.hasFontFileExtension(fontLocation)){
                    this.addFont(fontLocation);
                } else {
                    var locations = EFI.Utils.getAdobeResourceFontLocations(font);
                    if(locations){
                        for (var l = 0; l < locations.length; l++) {
                            this.addFont(locations[l]);    
                        }                        
                    }
                }
            }
            
        }
    
        // we also export unused text styles
        // SmartCanvas will complain later that fonts which are defined in the text styles are missing
        // even when they are not used in the document
        var documentTextStyles = app.activeDocument.paragraphStyles;
        
        for(var si=0; si<documentTextStyles.length; si++){
                var docStyle = documentTextStyles[si];
                var appliedFont = docStyle.appliedFont;

                try {
                    if(appliedFont && appliedFont.location && appliedFont.location !== ""){

                        if(EFI.Utils.hasFontFileExtension(appliedFont.location)){
                            this.addFont(appliedFont.location);
                        } else {
                            var locations = EFI.Utils.getAdobeResourceFontLocations(appliedFont);
                            if(locations){
                                for (var l = 0; l < locations.length; l++) {
                                    this.addFont(locations[l]);
                                }                        
                            }
                        }
                    }    
                } catch (error) {

                }            
        }
     
};

EFI.Exporter.prototype.addSwatches = function(XML) {
    mySwatches = app.activeDocument.swatches;
    var clrSpace = EFI.ConfigSettings.colorSpace;

    XML.BeginNode("ColorSwatches");
    
    for(var i = 1; i < mySwatches.length;i++)
    {
        var curSwatch = mySwatches[i];
        var swatchName = curSwatch.name;
        
        if(swatchName !== "" && swatchName != "Registration" && swatchName !== "Paper" && swatchName !== "Black"){
        
            XML.BeginNode("ColorSwatch");

            

            var colorMode = EFI.Utils.getColorSpaceString(curSwatch);
            var colorType = EFI.Utils.getColorModelString(curSwatch);
                        
            var colorString = "";
            
            if(colorType === "spot"){
                if(curSwatch.hasOwnProperty("tintValue")){
                    XML.Attrib("Name", curSwatch.name); // it has the tint value in the name
                } else {
                    XML.Attrib("Name", curSwatch.name + " 100%");
                }
                
                colorString = "swatch," + curSwatch.name + " 100%,spot," + curSwatch.name + ",100," + EFI.Utils.getColor(curSwatch);
            } else {
                XML.Attrib("Name", curSwatch.name);
                
                colorString = "swatch," + curSwatch.name + "," +  EFI.Utils.getColor(curSwatch);;
            }      
        
            XML.Attrib("Mode", colorMode);
            XML.Attrib("ColorType", colorType);
            
            XML.Attrib("ColorString", colorString);
       
            XML.EndNode(); //ColorSwatch
            
        }
    }
    XML.EndNode(); //ColorSwatches
};

function getTextStyleName(textStyle) {

    var name = "";

    if(textStyle) {

        if(typeof textStyle === "string") {
            name = textStyle;
        } else if(textStyle.name) {
            name = textStyle.name
        }

        if(name.length > 0){
            // now rename InDesign default style and remove brackets
            if(name.indexOf("[") === 0 && name.indexOf("]") === name.length-1){
                name = "InDesign Base Style"
            }
        }
    }

    return name;
}

EFI.Exporter.prototype.addTextStyles = function(XML) {
    //Add Paragraph and Character Styles
    var myTextStyles = app.activeDocument.paragraphStyles;
    var myCharStyles = app.activeDocument.characterStyles;

    XML.BeginNode("TextStyles");
    for(var i = 1; i < myTextStyles.length;i++)
    {
        var curTextStyle = myTextStyles[i];
        XML.BeginNode("ParagraphStyle");

        var styleName = getTextStyleName(curTextStyle);
        XML.Attrib("Name", styleName);
              
        XML.Attrib("BasedOn", styleName === "InDesign Base Style" ? "" : getTextStyleName(curTextStyle.basedOn));
      
        XML.Attrib("FontSize", curTextStyle.pointSize);

        XML.Attrib("LineHeight", curTextStyle.leading);
                
        if(curTextStyle.appliedFont && curTextStyle.appliedFont.status !== FontStatus.NOT_AVAILABLE){
            if(curTextStyle.appliedFont.hasOwnProperty("fontFamily")){
                XML.Attrib("FontFamily", curTextStyle.appliedFont.fontFamily);
            }
            if(curTextStyle.appliedFont.hasOwnProperty("fontStyleName")) {
                XML.Attrib("FontStyle", EFI.Utils.getFontStyle(curTextStyle.appliedFont.fontStyleName));

                XML.Attrib("FontWeight", EFI.Utils.getFontWeight(curTextStyle.appliedFont.fontStyleName));
            }
        }
    
        XML.Attrib("Foreground", EFI.Utils.getSwatchNameOrColor(curTextStyle.fillColor));
        
        var leftIndent = curTextStyle.leftIndent;
        
        if(curTextStyle.bulletsAndNumberingListType === ListType.BULLET_LIST && curTextStyle.bulletChar){
                
                if(curTextStyle.tabList.length > 0){
                    leftIndent = curTextStyle.tabList[0].position;
                }
                var character = String.fromCharCode(curTextStyle.bulletChar.characterValue);
                XML.Attrib("BulletCharacter", character);
        }
    
    
        if(curTextStyle.tracking === "0") XML.Attrib("Tracking", "normal");
        
        XML.Attrib("TextAlignment", EFI.Utils.getTextAlign(curTextStyle.justification));
        
        XML.Attrib("LeftIndent", leftIndent);
        XML.Attrib("RightIndent", curTextStyle.rightIndent);
        XML.Attrib("StartIndentOffset", curTextStyle.firstLineIndent);
                
        XML.Attrib("SpaceAfter", curTextStyle.spaceAfter);
        XML.Attrib("SpaceBefore", curTextStyle.spaceBefore);
        
        // RULE(s) = Line above and below an paragraph
        
        if(curTextStyle.ruleAbove){
                XML.Attrib("RuleAboveColor", EFI.Utils.getSwatchNameOrColor(curTextStyle.ruleAboveColor));
                XML.Attrib("RuleAboveOffset", curTextStyle.ruleAboveOffset);
                XML.Attrib("RuleAboveThickness", curTextStyle.ruleAboveLineWeight);
                XML.Attrib("RuleAboveLeftIndent", curTextStyle.ruleAboveLeftIndent);
                XML.Attrib("RuleAboveRightIndent", curTextStyle.ruleAboveRightIndent);                
        }
    
        if(curTextStyle.ruleBelow){
                XML.Attrib("RuleBelowColor", EFI.Utils.getSwatchNameOrColor(curTextStyle.ruleBelowColor));
                XML.Attrib("RuleBelowOffset", curTextStyle.ruleBelowOffset);
                XML.Attrib("RuleBelowThickness", curTextStyle.ruleBelowLineWeight);
                XML.Attrib("RuleBelowLeftIndent", curTextStyle.ruleBelowLeftIndent);
                XML.Attrib("RuleBelowRightIndent", curTextStyle.ruleBelowRightIndent);
        }
        
        XML.EndNode();
    }

     XML.EndNode(); //TextStyles

    /*
    for(var i = 1; i < myCharStyles.length;i++)
    {
        var curCharStyle = myCharStyles[i];
        XML.BeginNode("CharacterStyle");
        XML.Attrib("Name", myCharStyles.name);
        if(curCharStyle.appliedFont != "") XML.Attrib("FontFamily", curCharStyle.appliedFont);
        if(curCharStyle.pointSize != "") XML.Attrib("FontSize", curCharStyle.pointSize);
        XML.EndNode();
    }


    XML.EndNode(); //TextStyles


        alert(EFI.UIHelper.getVerticalAlign(curTextStyle));

        alert("1");
        //EFI.Utils.getParameters(curTextStyle);
        EFI.Utils.getParameters(curTextStyle.appliedFont);

        //EFI.Utils.getParameters(curTextStyle.appliedFont);
        //app.selection[0].textFramePreferences.verticalJustification





    <ParagraphStyle  Name="[Default]" IsDefaultStyle="true" FontFamily="Arial"
    FontSize="11" Tracking="normal" Foreground="swatch, [Black]" LineHeight="0"
    TextAlignment="left" SpaceAfter="0" SpaceBefore="0" RuleAboveMode="column"
    RuleAboveRef="top" RuleBelowRef="bottom" RuleBelowMode="column" />

    mySwatches = app.documents[0].swatches;
    app.activeDocument.paragraphStyles.item ("[Basic Paragraph]")
    var clrSpace = EFI.ConfigSettings.colorSpace;

    XML.BeginNode("ColorSwatches");
    for(var i = 1; i < mySwatches.length;i++)
    {
        var curSwatch = mySwatches[i];
        XML.BeginNode("ColorSwatch");
        XML.Attrib("Name", curSwatch.name);
        switch(curSwatch.space) {
            case clrSpace.CMYK:
                XML.Attrib("ColorString", "cmyk," + curSwatch.colorValue);
                break;
            case clrSpace.LAB:
                XML.Attrib("ColorString", "lab," + curSwatch.colorValue);
                break;
            case clrSpace.MIXEDINK:
                XML.Attrib("ColorString", "mixedink," + curSwatch.colorValue);
                break;
            case clrSpace.RGB:
                XML.Attrib("ColorString", "rgb," + curSwatch.colorValue);
                break;
            default:
                //do nothing
        }

        //XML.Attrib("WebColor", "");
        //="R=177 G=0 B=23" ="rgb,69.41,0,9.019" WebColor="#b10017"
        XML.EndNode(); //ColorSwatch
    }
    XML.EndNode(); //ColorSwatches
    */
};

//----------------//Exporter-----------