﻿
//-----------------FieldManager-----------------
//stores the overall fields that have been marked as having a variable in it

EFI.FieldManager = function () {
	this.currentSelectableObject = undefined;
	this.currentBoxProperties = undefined;
	//this.currentFieldProperties = undefined;
	this.currentTemplateProperties = undefined; //of type TemplateProperties
	this.VariableObjects = {}; //of type SelectableObjects
	this.cachedNativeFields = {}; //cache of native InDesign fields by Id
};

EFI.FieldManager.prototype.setUserField = function(propertiesObject, propertyName, propertyValue) {
	if(propertiesObject[propertyName]) propertiesObject[propertyName].fieldValue = propertyValue;
};

EFI.FieldManager.prototype.getUserFieldValue = function(propertiesObject, propertyName) {
	var fieldValue;

	if(propertiesObject[propertyName]) fieldValue = propertiesObject[propertyName].fieldValue;

	return fieldValue;
};

EFI.FieldManager.prototype.setUserFieldsAll = function(userFieldProperties) {
	alert("set user fields all??");
	//this.UserFieldProperties = userFieldProperties;
};

//don't think we need anymore as not field by field basis
/*EFI.FieldManager.prototype.getUserField = function(placeholder) {
	EFI.Utils.addDebugLog("EFI.FieldManager.prototype.getUserField - placeHolder: " + placeholder);
	var stringId = new String(placeholder);
	var userField = this.UserFieldProperties[stringId];

	if(!userField) {
		userField = new EFI.FieldProperties(placeholder);
		userField.placeHolder.fieldValue = placeholder;
	}

	return userField;
};

EFI.FieldManager.prototype.getUserFields = function() {
    var fields = [];

	for(var key in this.UserFieldProperties) {
        if(this.UserFieldProperties.hasOwnProperty(key)) fields.push(this.UserFieldProperties[key]);
    }

    return fields;
};*/

//arg: field
//of type SelectableObject
EFI.FieldManager.prototype.addVariableObject = function(selectableObject) {
	EFI.Utils.addDebugLog("EFI.FieldManager.prototype.addVariableObject - id: " + selectableObject.getId());
	var stringId = new String(selectableObject.getId());

	this.VariableObjects[stringId] = selectableObject;
};

EFI.FieldManager.prototype.getVariableObject = function(id) {
	var baseField = EFI.ObjectFactory.getInstance("fieldManager").getBaseObjectFromId(id);
	var variableObject = undefined;

	if(baseField) {
		var stringId = new String(id);
		variableObject = this.VariableObjects[stringId];
	}

	return variableObject;
};

EFI.FieldManager.prototype.removeVariableObject = function(id) {
	var stringId = new String(id);
	this.VariableObjects[stringId] = undefined;
};

/*EFI.FieldManager.prototype.getVariableObject = function(id) {
	var stringId = new String(id);

	return this.VariableObjects[stringId];
};*/

EFI.FieldManager.prototype.getVariableObjects = function() {
	var variableObjects = [];

	for(var key in this.VariableObjects) {
		if(this.VariableObjects.hasOwnProperty(key)) {
			var variableObject = this.getVariableObject(key);
			if(variableObject) variableObjects.push(variableObject);
		}
	}

	return variableObjects;
};

EFI.FieldManager.prototype.clearVariableObjects = function() {
	EFI.Utils.addDebugLog("EFI.FieldManager.prototype.clearVariableObjects");
	this.VariableObjects = {};
};

EFI.FieldManager.prototype.setVariableObjects = function(variableObjects) {
	this.VariableObjects = variableObjects;
};

EFI.FieldManager.prototype.clearAll = function() {
	EFI.Utils.addDebugLog("EFI.FieldManager.prototype.clearAll");
	this.clearVariableObjects();
};

//save the current field settings to InDesign document
EFI.FieldManager.prototype.saveState = function() {
	EFI.Utils.addDebugLog("EFI.FieldManager.prototype.saveState");

	//alert("save state? user field properties");
	EFI.Utils.saveObjectState(EFI.UIHelper.getCurrentDocument(), this.getTemplateDetailsObject(), EFI.ConfigSettings.globals.objectTemplateSaveLabel);
};

//read field settings from InDesign document
/*EFI.FieldManager.prototype.getFieldPropertiesState = function() {
	return EFI.Utils.getObjectState(EFI.UIHelper.getCurrentDocument(), EFI.ConfigSettings.globals.objectFieldPropertiesSaveLabel);
};*/

//read template settings from InDesign document
//param: testMocInjectionField is used for unit testing only
EFI.FieldManager.prototype.getTemplateState = function(testMocInjectionField) {
	var loadedField = EFI.Utils.getObjectState(EFI.UIHelper.getCurrentDocument(), EFI.ConfigSettings.globals.objectTemplateSaveLabel);
	if(loadedField === undefined) loadedField = {};

	var newField = testMocInjectionField;
	if(newField === undefined) newField = new EFI.TemplateProperties();

	//copy over saved settings into new fresh object. In case new properties have been added since document saved
	EFI.Utils.copyObjectFieldValueProperties(loadedField.templateProperties, newField);

	loadedField.templateProperties = newField;

	return loadedField;
};

EFI.FieldManager.prototype.getTemplateDetailsObject = function() {
	var templateDetails = {};
	var saveFields = [];

	var aryVariableFields = this.getVariableObjects();

	for (var i = 0; i < aryVariableFields.length; i++) {
		var field = aryVariableFields[i].getBoxProperties();

		saveFields.push(field);
	}

	templateDetails.boxProperties = saveFields;
	templateDetails.templateProperties = this.currentTemplateProperties;

	return templateDetails;
};

EFI.FieldManager.prototype.loadInitialSavedData = function() {
	EFI.Utils.addDebugLog("EFI.FieldManager.prototype.loadInitialSavedData");

	try {
		var templateSettings = this.getTemplateState();

		if(templateSettings) {
			EFI.Utils.addDebugLog("EFI.FieldManager.prototype.loadInitialSavedData - has templateSettings: " + templateSettings);
			EFI.Utils.addDebugLog("EFI.FieldManager.prototype.loadInitialSavedData - JSON: " + EFI.Utils.getJSON(templateSettings));


			this.setCurrentTemplateProperties(templateSettings.templateProperties);

			if(templateSettings.boxProperties) {
				EFI.Utils.addDebugLog("EFI.FieldManager.prototype.loadInitialSavedData - number variable fields: " + templateSettings.boxProperties.length);

				for(var i=0; i < templateSettings.boxProperties.length; i++) {
					var selectableObject;
					var boxProperties = new EFI.BoxProperties("", templateSettings.boxProperties[i]);
					var boxType = boxProperties.getBoxType();
					var nativeField = EFI.UIHelper.getItemById(boxProperties.getId());

					if(nativeField) {
						if(boxType === "Multi Field" || boxType === "Text Object") {
							selectableObject = this.processTextFieldRules(nativeField, boxType);
							//EFI.Utils.addDebugLog("EFI.FieldManager.prototype.loadInitialSavedData - fields loaded: " + selectableObject.getTextBlock().getFields().length);
						} else {
							selectableObject = EFI.ObjectFactory.getInstance("selectableObject", nativeField);
						}

						selectableObject.setBoxProperties(boxProperties);
					}
				}
			}
		} else {
			this.setCurrentTemplateProperties(new EFI.TemplateProperties());
			EFI.Utils.addDebugLog("EFI.FieldManager.prototype.loadInitialSavedData - templateSettings NOT found");
		}

	} catch(error) {
		EFI.Utils.addDebugLog(error);
	}
};

//param: textField is base InDesign textField (selectableObject.getBaseObject())
EFI.FieldManager.prototype.processTextFieldRules = function (textField, boxType) {
	var multiline = false;
	var theBoxType = boxType;

	if (!theBoxType || theBoxType === "") {
		var selectableObject = EFI.ObjectFactory.getInstance("selectableObject", textField);
		theBoxType = selectableObject.getBoxProperties().getBoxType();
	}

	if (theBoxType.toLowerCase() === "multi field" || theBoxType.toLowerCase() === "text object") {
		if (theBoxType.toLowerCase() === "text object") multiline = true;

		var ruleEngine = new EFI.RuleEngine(textField, multiline);
		return ruleEngine.processFieldRules();
	} else {
		return undefined;
	}
};

/*EFI.FieldManager.prototype.setCurrentFieldProperties = function(fieldProperties) {
	this.currentFieldProperties = fieldProperties;
}

EFI.FieldManager.prototype.getCurrentFieldProperties = function() {
	return this.currentFieldProperties;
}*/

EFI.FieldManager.prototype.setCurrentBoxProperties = function (boxProperties) {
	this.currentBoxProperties = boxProperties;
};

EFI.FieldManager.prototype.getCurrentSelectableObject = function () {
	return this.currentSelectableObject;
};

EFI.FieldManager.prototype.setCurrentSelectableObject = function (selectableObject) {
	this.currentSelectableObject = selectableObject;
};

EFI.FieldManager.prototype.getCurrentBoxProperties = function () {
	return this.currentBoxProperties;
};

/*EFI.FieldManager.prototype.saveCurrentFieldProperties = function() {
	EFI.UserControlMapper.doMapping("FieldProperties", "SaveControls");
	this.saveState();
}*/

EFI.FieldManager.prototype.saveCurrentBoxProperties = function () {
	EFI.UserControlMapper.doMapping("BoxProperties", "SaveControls");
	this.saveState();
};

EFI.FieldManager.prototype.setCurrentTemplateProperties = function (templateProperties) {
	this.currentTemplateProperties = templateProperties;
};

EFI.FieldManager.prototype.getCurrentTemplateProperties = function () {
	var xOrig = EFI.UIHelper.switchXToPoints();
	var yOrig = EFI.UIHelper.switchYToPoints();

    /* AK 2017
	this.currentTemplateProperties.documentHeight.fieldValue = EFI.Utils.getDocumentHeight();
	this.currentTemplateProperties.documentWidth.fieldValue = EFI.Utils.getDocumentWidth();
	*/
	/*this.currentTemplateProperties.bleedTop.fieldValue = EFI.Utils.getBleedTop();
	this.currentTemplateProperties.bleedBottom.fieldValue = EFI.Utils.getBleedBottom();
	this.currentTemplateProperties.bleedLeft.fieldValue = EFI.Utils.getBleedLeft();
	this.currentTemplateProperties.bleedRight.fieldValue = EFI.Utils.getBleedRight();*/

	EFI.UIHelper.switchXToOriginal(xOrig);
	EFI.UIHelper.switchYToOriginal(yOrig);

	return this.currentTemplateProperties;
};

EFI.FieldManager.prototype.getTemplateType = function () {
	return '';
};

EFI.FieldManager.prototype.saveCurrentTemplateProperties = function () {
	EFI.UserControlMapper.doMapping("Template", "SaveControls");
	this.saveState();
};

EFI.FieldManager.prototype.getBaseObjectFromId = function (id) {
	var fieldWanted;

	var field = EFI.UIHelper.getItemById(id);

	if (field) {
		//if sub image selected, we want the parent image frame
		if (field.constructor.name.toLowerCase() === EFI.ConfigSettings.objectTypes.image) {
			fieldWanted = field.parent;
		} else {
			fieldWanted = field;
		}
	}

	return fieldWanted;
};

EFI.FieldManager.prototype.askAddAlphabet = function () {
	var eventMediator = EFI.ObjectFactory.getInstance("eventMediator");
	var myWindow = new Window("dialog", "Add Character Set");
	myWindow.orientation = "row";
	myWindow.add("statictext", undefined, "Would you like to add the Character set to your field?");
	var yesButton = myWindow.add("button", undefined, "Yes", { name: "ok" });
	var noButton = myWindow.add("button", undefined, "No", { name: "cancel" });

	if (myWindow.show() === 1) {
		this.addCharacterSet();
	} else {
		exit();
	}
};

EFI.FieldManager.prototype.addCharacterSet = function () {
	var currentSelectedObject = this.getCurrentSelectableObject();

	if (currentSelectedObject.isTextFrame) {
		var textFrame = currentSelectedObject.getBaseObject();
		EFI.Utils.addCharacterSet(textFrame);
	}
};
//-----------------//FieldManager-----------------