﻿
//----------------XMLEngine-------------
//Generates XML for fields

EFI.GeneratorTextFormat = function(textFrame) {
	this.textFrame = textFrame;
	this.textObject = {};
};

EFI.GeneratorTextFormat.prototype.generateFormatting = function() {
	var xOrig = EFI.UIHelper.switchXToPoints();
	var yOrig = EFI.UIHelper.switchYToPoints();

	this.getParagraph();
	this.XML = this.addDoc();

	EFI.UIHelper.switchXToOriginal(xOrig);
	EFI.UIHelper.switchYToOriginal(yOrig);

	return this.XML;
};

EFI.GeneratorTextFormat.prototype.getParagraph = function() {
	var paragraphs = this.textFrame.paragraphs;

	EFI.Utils.addDebugLog("Paragraphs: " + paragraphs.length);
	for(var i = 0; i < paragraphs.length; i++) {
		EFI.Utils.addDebugLog("Paragraph Content: " + paragraphs[i].contents);
		if(paragraphs[i]) this.addParagraph(paragraphs[i]);
	}
};

EFI.GeneratorTextFormat.prototype.addParagraph = function(paragraph) {
	this.currentParagraph.totalChars = paragraph.contents.length;
	EFI.Utils.addDebugLog("Paragraph Chars: " + this.currentParagraph.totalChars);

	this.XML += "<Paragraph FontSize='" + paragraph.pointSize + "' FontFamily='" + paragraph.appliedFont.fullName + "' FontStyle='Regular' FontWeight='" + paragraph.fontStyle + "' TextAlignment='" + this.getAlignment(paragraph) + "' LineHeight='" + this.getLineHeight(paragraph) + "'>";
	var styleXML = this.getStyle(paragraph);
	this.XML += "</Paragraph>";
}

EFI.GeneratorTextFormat.prototype.getStyle = function(paragraph) {
	var textStyleRange = paragraph.textStyleRanges;

	EFI.Utils.addDebugLog("Styles in Paragraph: " + textStyleRange.length);

	for(var i = 0; i < textStyleRange.length; i ++) {
		if(textStyleRange[i]) {
			//stop processing if the style is beyond the Paragraph
			if (this.addStyleXML(textStyleRange[i])) break;
		}
	}
};

//returns if it should stop processing more styles
EFI.GeneratorTextFormat.prototype.addStyle = function(style) {
	var contents = style.contents;
	var charsOverParagraph = (contents.length + this.currentParagraph.textPosition) - this.currentParagraph.totalChars;
	var stopProcessing = false;

	EFI.Utils.addDebugLog("Style Contents: " + contents);

	//Sometimes the Style Range goes beyond the Paragraph, so we need to trim it to just this paragraph
	if(charsOverParagraph > 0) {
		contents = contents.substring(0, contents.length - charsOverParagraph);
		stopProcessing = true;
		EFI.Utils.addDebugLog("Trimmed Contents: " + contents);
	}

	//InDesign adds each new line as a Paragraph, which already does a line break
	if (contents.charAt(contents.length -1) === "\r") {
		contents = contents.substring(0, contents.length - 1);
		EFI.Utils.addDebugLog("Trim carriage return: " + contents);
	}

	if(contents !== "") {
		this.XML += "<Run FontSize='" + style.pointSize + "' FontFamily='" + style.appliedFont.fullName + "' Foreground='" + this.getColor(style.fillColor) + "' FontStyle='Regular' FontWeight='" + style.fontStyle + "'>";
		this.XML += contents;
		this.XML += "</Run>";
	}

	return stopProcessing;
};

EFI.GeneratorTextFormat.prototype.addDoc = function() {
	//Not sure which one to use
	//var bounds = this.textFrame.geometricBounds;
	var bounds = this.textFrame.visibleBounds;
	var y1 = bounds[0];
	var x1 = bounds[1];
	var y2 = bounds[2];
	var x2 = bounds[3];

	var width = x2 - x1;
	var height = y2 - y1;

	var curDoc = EFI.UIHelper.getCurrentDocument();

	var bodyXML = "<Composition Height='" + curDoc.documentPreferences.pageHeight + "' Width='" + curDoc.documentPreferences.pageWidth + "' Version='1'>";
	bodyXML += "<Text Left='" + x1 + "' Top='" + y1 + "' Width='" + width + "' Height='" + height + "' Rotation='0.0'>";
	bodyXML += this.XML;
	bodyXML += "</Text>";
	bodyXML += "</Composition>";

	return bodyXML;
};

//----------------//XMLEngine-----------

EFI.GeneratorTextFormat.Paragraph = function (nativeParagraph) {
	this.currentParagraph = {};
	this.currentParagraph.textPosition = 0;
	this.currentParagraph.totalChars = 0;

	this.nativeParagraph = nativeParagraph;
	this.processFormatting();
};

EFI.GeneratorTextFormat.Paragraph.prototype.processFormatting = function () {
	this.fontSize = EFI.TextFormatHelper.getFontSize(this.nativeParagraph);
	this.alignment = EFI.TextFormatHelper.getAlignment(this.nativeParagraph);
	this.fontFamily = EFI.TextFormatHelper.getFontName(this.nativeParagraph);
	this.fontStyle = EFI.TextFormatHelper.getFontStyle(this.nativeParagraph);
	this.lineHeight = EFI.TextFormatHelper.getLineHeight(this.nativeParagraph);
	this.text = [];

	this.processText();
};

EFI.GeneratorTextFormat.Paragraph.prototype.processText = function () {
	var textStyleRange = this.nativeParagraph.textStyleRanges;

	EFI.Utils.addDebugLog("Styles in Paragraph: " + textStyleRange.length);

	for (var i = 0; i < textStyleRange.length; i++) {
		if (textStyleRange[i]) {
			//stop processing if the style is beyond the Paragraph
			if (this.addStyle(textStyleRange[i])) break;
		}
	}
};

EFI.GeneratorTextFormat.Paragraph.prototype.addStyle = function (textStyle) {
	var contents = textStyle.contents;
	var charsOverParagraph = (contents.length + this.currentParagraph.textPosition) - this.currentParagraph.totalChars;
	var stopProcessing = false;

	EFI.Utils.addDebugLog("Style Contents: " + contents);

	if (charsOverParagraph > 0) {
		contents = contents.substring(0, contents.length - charsOverParagraph);
		stopProcessing = true;
		EFI.Utils.addDebugLog("Trimmed Contents: " + contents);
	}

	this.text.push(new EFI.GeneratorTextFormat.Text(textStyle, contents));
};

EFI.GeneratorTextFormat.Text = function (textStyle, contents) {
	this.nativeTextStyle = textStyle;
	this.text = contents;

	this.processTextStyle();
};

EFI.GeneratorTextFormat.Text.prototype.processTextStyle = function () {
};

//---------------------GeneratorTextFormat--------------------