
//-----------------ObjectFactory-----------------
//Singleton loader for PluginUI and FieldManager, which can only have 1 live object state

EFI.ObjectFactory = {
	instancePluginUI: undefined, //of PluginUI
	instanceFieldManager: undefined, //of FieldManager
	instanceEventMediator: undefined, //of EventMediator

	init: function(type, theObject) {
		var returnItem;

		switch (type) {
			case "pluginui":
				returnItem = new EFI.PluginUI();
				break;
			case "fieldmanager":
				returnItem = new EFI.FieldManager();
				break;
			case "eventmediator":
				returnItem = new EFI.EventMediator();
				break;
			case "selectableobject":
				//return existing SelectableObject if already created, else create a new one

				if(theObject) {
					if (EFI.Utils.hasProperty(theObject, "id")) returnItem = this.getInstance("fieldManager").getVariableObject(theObject.id);
				}

				if(!returnItem) {
					returnItem = new EFI.SelectableObject(theObject);
				}
				break;
		}

		return returnItem;
	},

	// Get the Singleton instance if one exists
	// or create one if it doesn't
	getInstance: function(type, theObject) {
		type = type.toLowerCase();
		var returnInstance;

		switch (type) {
			case "pluginui":
				if ( !this.instancePluginUI) {
					this.instancePluginUI = this.init("pluginui");
					this.instancePluginUI.initialize();
				}

				returnInstance = this.instancePluginUI;
				//returnInstance.makeVisible();
				break;
			case "fieldmanager":
				if ( !this.instanceFieldManager ) this.instanceFieldManager = this.init("fieldmanager");

				returnInstance = this.instanceFieldManager;
				break;
			case "eventmediator":
				if ( !this.instanceEventMediator ) this.instanceEventMediator = this.init("eventmediator");

				returnInstance = this.instanceEventMediator;
				break;
			case "selectableobject":
				returnInstance = this.init("selectableobject", theObject);
				break;
		}

		return returnInstance;
	},

	getCurrentState: function(type) {
		type = type.toLowerCase();
		var returnInstance;

		switch (type) {
			case "pluginui":
				returnInstance = this.instancePluginUI;
				break;
			case "fieldmanager":
				returnInstance = this.instanceFieldManager;
				break;
			case "eventmediator":
				returnInstance = this.instanceEventMediator;
				break;
		}

		return returnInstance;
	}
};

//-----------------//ObjectFactory-----------------