﻿
//----------------PDFEngine-------------
//Generates PDF layers

EFI.PDFEngine = function() {
};

//param: file of type File
EFI.PDFEngine.prototype.exportPDF = function(folder, file, profile) {
    if (profile) {

        var oldPref = app.pdfExportPreferences.viewPDF;
        app.pdfExportPreferences.viewPDF = false; // Otherwise each layer pdf will be opened in the PDF reader
    
        var doc = EFI.UIHelper.getCurrentDocument();

        var efiPdfExportPreset = this.getOrCreateEFIPdfExportPreset();
        
        EFI.Utils.createCheckFolder(folder);
        doc.exportFile(ExportFormat.pdfType, file, false, efiPdfExportPreset);        
        
        // restore user setting
        app.pdfExportPreferences.viewPDF = oldPref;
    }
};

EFI.PDFEngine.prototype.pdfExportPresetName = "EFI-PDFPreset";

EFI.PDFEngine.prototype.getEFIPdfExportPreset = function () {
    
    for (var i = 0; i < app.pdfExportPresets.length; i++) {
        var preset = app.pdfExportPresets[i];
        if (preset && preset.name === this.pdfExportPresetName) {
            return preset;
        }
    }

    return null;
};

// we need to make sure, that we have a custom PDF export profile
// if we cannot find it, we will create it
EFI.PDFEngine.prototype.getOrCreateEFIPdfExportPreset = function () {
    
    var efiPreset = this.getEFIPdfExportPreset();

    if (efiPreset !== null) {
        return efiPreset;

    } else {

        // create EFI standard PDF Export Preset
        var newEFIPreset = app.pdfExportPresets.add();

        // rename it
        newEFIPreset.name = this.pdfExportPresetName;

        // set our default values

        newEFIPreset.acrobatCompatibility = AcrobatCompatibility.ACROBAT_5;
        newEFIPreset.standardsCompliance = PDFXStandards.NONE;

        //  GENERAL
        newEFIPreset.exportNonprintingObjects = false;
        newEFIPreset.exportAsSinglePages = false;
        newEFIPreset.exportWhichLayers = ExportLayerOptions.EXPORT_VISIBLE_PRINTABLE_LAYERS;
        newEFIPreset.optimizePDF = true;

        // COMPRESSION
        newEFIPreset.colorBitmapSampling = Sampling.BICUBIC_DOWNSAMPLE;
        newEFIPreset.colorBitmapSamplingDPI = 300;
        newEFIPreset.thresholdToCompressColor = 450;

        newEFIPreset.colorBitmapCompression = BitmapCompression.AUTO_COMPRESSION;
        newEFIPreset.colorBitmapQuality = CompressionQuality.MAXIMUM;

        newEFIPreset.grayscaleBitmapSampling = Sampling.BICUBIC_DOWNSAMPLE;
        newEFIPreset.grayscaleBitmapSamplingDPI = 300;
        newEFIPreset.thresholdToCompressGray = 450;
        newEFIPreset.grayscaleBitmapCompression = BitmapCompression.AUTO_COMPRESSION;
        newEFIPreset.grayscaleBitmapQuality = CompressionQuality.MAXIMUM;

        newEFIPreset.monochromeBitmapSampling = Sampling.BICUBIC_DOWNSAMPLE;
        newEFIPreset.monochromeBitmapSamplingDPI = 1200;
        newEFIPreset.thresholdToCompressMonochrome = 1800;
        newEFIPreset.monochromeBitmapCompression = MonoBitmapCompression.CCIT4;

        newEFIPreset.compressTextAndLineArt = true;
        newEFIPreset.cropImagesToFrames = true;

        // MARKS AND BLEEDS
        newEFIPreset.pageMarksOffset = 0.5;
        newEFIPreset.cropMarks = false;
        newEFIPreset.pageInformationMarks = false;
        newEFIPreset.bleedMarks = false;
        newEFIPreset.colorBars = false;
        newEFIPreset.registrationMarks = false;
        newEFIPreset.printerMarkWeight = PDFMarkWeight.P25PT;
        /*newEFIPreset.bleedTop = EFI.Utils.getBleedTop();
        newEFIPreset.bleedInside = EFI.Utils.getBleedLeft();
        newEFIPreset.bleedBottom = EFI.Utils.getBleedBottom();
        newEFIPreset.bleedOutside = EFI.Utils.getBleedRight();*/
        newEFIPreset.pdfMarkType = MarkTypes.DEFAULT_VALUE;
        newEFIPreset.useDocumentBleedWithPDF = true;
        newEFIPreset.includeSlugWithPDF = false;

        // OUTPUT
        newEFIPreset.pdfColorSpace = PDFColorSpace.UNCHANGED_COLOR_SPACE;
        newEFIPreset.includeICCProfiles = ICCProfiles.INCLUDE_TAGGED;

        // ADVANCED
        newEFIPreset.subsetFontsBelow = 100;
        newEFIPreset.omitEPS = false;
        newEFIPreset.omitPDF = false;
        newEFIPreset.omitBitmaps = false;

        return newEFIPreset;

    }
};

//----------------//PDFEngine-----------