﻿
//-----------------PluginUI-----------------
//Controls the UI menu

/*
TODO: Template Tab controls
TODO: Save/Load Template Tab controls
TODO: Able to name SelectleObject on creation
TODO: Generate XML output
TODO: Generate PDFs
TODO: Handle font embedded, subset add alphabet?
TODO: Save Images
TODO: Generate "ZIP Package"
TODO: Clean up for test deployment
TODO: Minimize/compile for production deployment
TODO: Test with multiple documents open
 */
EFI.PluginUI = function () {
	//TODO: Initially load all fields from RuleEngine
	//TODO: Save overall template details
	//TODO: Save variable fields/name

	this.createMenuObject();
};

EFI.PluginUI.prototype.createMenuObject = function() {
	this.menuObject = EFI.UIHelper.createPalette(EFI.ConfigSettings.globals.windowName);
	this.menuObject.window.preferredSize = [EFI.ConfigSettings.globals.windowWidth, EFI.ConfigSettings.globals.windowHeight];
};

EFI.PluginUI.prototype.initialize = function(objectInfo) {
	EFI.ObjectFactory.getInstance("fieldManager").loadInitialSavedData();
	this.buildUI();
	EFI.ObjectFactory.getInstance("eventMediator").initializeEvents();
	this.showPlugin();
};

//param: field of type selectedObject
EFI.PluginUI.prototype.setNewFieldState = function(selectedObject) {
	var boxProperties;

	if(selectedObject) boxProperties = selectedObject.boxProperties;
//if(boxProperties) EFI.Utils.getParameters(boxProperties.boxVariable);
	EFI.ObjectFactory.getInstance("fieldManager").setCurrentBoxProperties(boxProperties);

	this.setupFieldsTab();
};

//param: fieldProperties of type FieldProperties
EFI.PluginUI.prototype.setNewPropertiesState = function(fieldProperties) {
	var fieldManager = EFI.ObjectFactory.getInstance("fieldManager");

	fieldManager.setCurrentFieldProperties(fieldProperties);

	this.setupPropertiesTab();
	this.setupInfoTab(fieldProperties);
};

EFI.PluginUI.prototype.setupFieldsTab = function() {
	var fieldTab = this.getMenuObject(EFI.ConfigSettings.menuObjectNames.fields);

	EFI.UserControlMapper.doMapping("BoxProperties", "AddControls");

	//TODO: Move Save button outside panel
	//TODO: Add cancel button
	//TODO: On move off tab, confirm they will lose details
	//var saveButton = EFI.UIHelper.addButton(fieldTab, "Save");
	//EFI.ObjectFactory.getInstance("eventMediator").addOnClickSaveFields(saveButton);

	EFI.UIHelper.refreshLayout(fieldTab);
};

EFI.PluginUI.prototype.setupPropertiesTab = function() {
	var propertyTab = this.getMenuObject(EFI.ConfigSettings.menuObjectNames.properties);

	EFI.UserControlMapper.doMapping("FieldProperties", "AddControls");

	//TODO: Move Save button outside panel
	//TODO: Add cancel button
	//TODO: On move off tab, confirm they will lose details
	//var saveButton = EFI.UIHelper.addButton(propertyTab, "Save");
	//EFI.ObjectFactory.getInstance("eventMediator").addOnClickSaveProperty(saveButton);

	EFI.UIHelper.refreshLayout(propertyTab);
};

EFI.PluginUI.prototype.setupTemplateTab = function() {
	var templateTab = this.getMenuObject(EFI.ConfigSettings.menuObjectNames.template);

	EFI.UserControlMapper.doMapping("Template", "AddControls");

	//debug buttons
	/*var saveButton = EFI.UIHelper.addButton(templateTab, "Save");
	EFI.ObjectFactory.getInstance("eventMediator").addOnClickSaveTemplate(saveButton);

	var clearState = EFI.UIHelper.addButton(templateTab, "Clear State");
	EFI.ObjectFactory.getInstance("eventMediator").addOnClickClearState(clearState);

	var checkText = EFI.UIHelper.addButton(templateTab, "Check Text");
	EFI.ObjectFactory.getInstance("eventMediator").addOnClickCheckText(checkText);*/

	var exportTemplate = EFI.UIHelper.addButton(templateTab, "Export as SmartCanvas Document");
	EFI.ObjectFactory.getInstance("eventMediator").addOnClickExportTemplate(exportTemplate);

	EFI.UIHelper.refreshLayout(templateTab);
};

//TODO: Add all info to tab
//TODO: Add up arrow/graphic to show relatives
//param: field of type Field
EFI.PluginUI.prototype.setupInfoTab = function(field) {
	var infoTab = this.getMenuObject(EFI.ConfigSettings.menuObjectNames.fieldInfo);
	var fieldManager = EFI.ObjectFactory.getInstance("fieldManager");

	EFI.UIHelper.removeChildObjects(infoTab);

	if(field) {
		EFI.UIHelper.addStaticText(infoTab, "Placeholder: " + field.placeHolder);

		if(field.relativeField) {
			EFI.UIHelper.addStaticText(infoTab, "Relative To: " + field.relativeField.placeHolder);
			EFI.UIHelper.addStaticText(infoTab, "Direction: " + field.relativeDirection);
		} else {
			EFI.UIHelper.addStaticText(infoTab, "x: " + field.x);
			EFI.UIHelper.addStaticText(infoTab, "y: " + field.y);
		}
	}

	EFI.UIHelper.refreshLayout(infoTab);
};

EFI.PluginUI.prototype.showPlugin = function() {
	this.makeVisible();
};

EFI.PluginUI.prototype.buildUI = function() {
	var w = this.getWindow();
	w.alignChildren = "right";

	//add tabbar container
	var tabBar = EFI.UIHelper.addTabPanel(w, "mainTabs");
	tabBar.alignChildren = ["fill", "fill"];
	tabBar.preferredSize = [EFI.ConfigSettings.globals.windowWidth, EFI.ConfigSettings.globals.windowHeight];
	w.tabBar = tabBar;

	//add template tab
	var templateTab = EFI.UIHelper.addTab(tabBar, "Document Details", "templateTab");
	w.templateTab = templateTab;
	templateTab.alignChildren = "fill";

	var templateGeneralPanel = EFI.UIHelper.addPanel(templateTab, "General", "templateGeneralPanel");
	templateGeneralPanel.alignChildren = "left";
	this.setupTemplateTab();

	//add fields tab
	var fieldsTab = EFI.UIHelper.addPanel(templateTab, "Field Details", "fieldsTab");
	w.fieldsTab = fieldsTab;
	fieldsTab.preferredSize = [EFI.ConfigSettings.globals.fieldPanelWidth, EFI.ConfigSettings.globals.fieldPanelHeight];
	fieldsTab.alignChildren = "fill";

	//w.fieldsTree = EFI.UIHelper.addTreeView(fieldsTab, EFI.ConfigSettings.menuObjectNames.fieldsTree);

	EFI.UIHelper.refreshLayout(templateTab);
};

EFI.PluginUI.prototype.makeVisible = function() {
	this.getWindow().show();
};

EFI.PluginUI.prototype.getWindow = function() {
	return this.menuObject.window;
};

EFI.PluginUI.prototype.getMenuObject = function(objectName) {
	var foundObject;
	var w = this.getWindow();
	
	switch(objectName.toLowerCase()) {
		case EFI.ConfigSettings.menuObjectNames.tabBar:
			foundObject = w.tabBar;
			break;
		case EFI.ConfigSettings.menuObjectNames.template:
			foundObject = w.templateTab;
			break;
		case EFI.ConfigSettings.menuObjectNames.fields:
			foundObject = w.fieldsTab;
			break;
	}

	return foundObject;
};

EFI.PluginUI.prototype.selectTab = function(tabName, fieldId) {
	var tabToSelect;

	tabToSelect = this.getMenuObject(tabName);
	
	if(tabToSelect) this.getMenuObject(EFI.ConfigSettings.menuObjectNames.tabBar).selection = tabToSelect;
};

EFI.PluginUI.prototype.updateFieldList = function() {
	var fieldManager = EFI.ObjectFactory.getInstance("fieldManager");
	var eventMediator = EFI.ObjectFactory.getInstance("eventMediator");
	var fieldsTab = this.getMenuObject(EFI.ConfigSettings.menuObjectNames.fields);
	//var fieldsTree = this.getMenuObject(EFI.ConfigSettings.menuObjectNames.fieldsTree);
	var selectedObject = fieldManager.getCurrentSelectableObject();

	/*if(fieldsTree == undefined) {
		EFI.UIHelper.addTreeView(fieldsTab, EFI.ConfigSettings.menuObjectNames.fieldsTree);
		eventMediator.initializeTreeEvents();
	} else {
		EFI.UIHelper.removeChildObjects(fieldsTree);
	}*/

	this.addFieldPanel(selectedObject);

	EFI.UIHelper.refreshLayout(fieldsTab);
};

EFI.PluginUI.prototype.addFieldPanel = function(selectableObject) {
	EFI.Utils.addDebugLog("EFI.PluginUI.prototype.addFieldPanel - id: " + selectableObject.getId());
	/*var fieldsTree = this.getMenuObject(EFI.ConfigSettings.menuObjectNames.fieldsTree);
	var node = EFI.UIHelper.addTreeViewNode(fieldsTree, selectableObject.getId());

	var aryFields = selectableObject.getTextBlock().getFields();
	EFI.Utils.addDebugLog("EFI.PluginUI.prototype.addFieldPanel - fields: " + aryFields.length);
	aryFields.reverse();

	for(var i = 0; i < aryFields.length; i++) {
		var newItem = EFI.UIHelper.addTreeViewItem(node, aryFields[i].placeHolder, aryFields[i].placeHolder);
		newItem.efiSelectableObjectId = selectableObject.getId();
		newItem.efiFieldNumber = aryFields.length - i;
	}

	node.expanded = true;*/
};

EFI.PluginUI.prototype.getSelectedTab = function() {
	return this.getMenuObject(EFI.ConfigSettings.menuObjectNames.tabBar).selection;
};

//-----------------//PluginUI-----------------