
//-----------------UIHelper-----------------
//Functions that help control/build/interact with InDesign UI

EFI.UIHelper = {
	addUserControl: function (menuObject, userControl, menuType) {
		var field;
		var eventMediator = EFI.ObjectFactory.getInstance("eventMediator");
		var fieldManager = EFI.ObjectFactory.getInstance("fieldManager");

		switch (userControl.fieldType) {
			case EFI.ConfigSettings.userControlTypes.textbox:
				field = EFI.UIHelper.addTextfield(menuObject, userControl.userCaption, userControl.fieldValue, userControl.id);
				if (menuType === "boxproperties") {
					//eventMediator.onTextChangeSaveFields(field);
				} else if (menuType === "template") {
					eventMediator.onTextChangeSaveTemplate(field);
				}

				break;
			case EFI.ConfigSettings.userControlTypes.readonly:
				field = EFI.UIHelper.addReadOnly(menuObject, userControl.userCaption, userControl.fieldValue, userControl.id);
				break;
			case EFI.ConfigSettings.userControlTypes.checkbox:
				field = EFI.UIHelper.addCheckbox(menuObject, userControl.userCaption, userControl.fieldValue, userControl.id);
				if (menuType === "boxproperties") {
					eventMediator.addOnClickCheckText(field);
				}
				break;
			case EFI.ConfigSettings.userControlTypes.dropdown:
				var options = userControl.options;

				if (menuType === "boxproperties") {
					if (fieldManager.getCurrentSelectableObject().isImageFrame()) {
						options = EFI.ConfigSettings.boxTypes.arrayBoxTypeImage;
					} else {
						options = EFI.ConfigSettings.boxTypes.arrayBoxTypeText;
					}
				}

				field = EFI.UIHelper.addDropDown(menuObject, userControl.userCaption, userControl.fieldValue, userControl.id, options, userControl.required);
				if (menuType === "boxproperties") {
					eventMediator.onDropDownChangeSaveFields(field);
				} else if (menuType === "template") {
					eventMediator.onDropDownChangeSaveTemplate(field);
				}

				break;
			case EFI.ConfigSettings.userControlTypes.pdfExportPreference:
				var pdfPreferences = EFI.Utils.getPDFExportPreferences();
				field = EFI.UIHelper.addDropDown(menuObject, userControl.userCaption, userControl.fieldValue, userControl.id, pdfPreferences, userControl.required);
				eventMediator.onDropDownChangeSaveTemplate(field);
				break;
		}

		menuObject["efi" + userControl.id] = field;

		return field;
	},

	getUserControlValue: function (menuObject, userControl) {
		var field = menuObject["efi" + userControl.id];
		var controlValue = undefined;

		switch (userControl.fieldType) {
			case EFI.ConfigSettings.userControlTypes.textbox:
				controlValue = field.text;
				break;
			case EFI.ConfigSettings.userControlTypes.readonly:
				controlValue = field.text;
				break;
			case EFI.ConfigSettings.userControlTypes.checkbox:
				controlValue = field.value;
				break;
			case EFI.ConfigSettings.userControlTypes.dropdown:
				if (field.selection) {
					controlValue = field.selection.text;
				} else {
					controlValue = "";
				}

				break;
			case EFI.ConfigSettings.userControlTypes.pdfExportPreference:
				if (field.selection) {
					controlValue = field.selection.text;
				} else {
					controlValue = "";
				}

				break;
		}

		return controlValue;
	},

	addStaticText: function (menuObject, caption) {
		return menuObject.add("statictext", undefined, caption);
	},

	addHiddenText: function (menuObject, defaultValue, itemName) {
		var hiddenField = this.addTextfield(menuObject, "", defaultValue, itemName);
		hiddenField.visible = false;
		hiddenField.size = [0, 0];

		return hiddenField;
	},

	addReadOnly: function (menuObject, caption, defaultValue, itemName) {
		this.addStaticText(menuObject, caption + ": " + defaultValue);
		return this.addHiddenText(menuObject, defaultValue, itemName);
	},

	addCheckbox: function (menuObject, caption, defaultValue, itemName) {
		//return menuObject.checkbox(name, caption);
		var theItem = menuObject.add("checkbox", undefined, caption, { name: itemName });
		if (defaultValue === true) theItem.value = true;

		return theItem;
	},

	addDropDown: function (menuObject, caption, defaultValue, itemName, optionList, required) {
		if (caption !== "") this.addStaticText(menuObject, caption);
		var theItem = menuObject.add("dropdownlist", undefined, optionList, { name: itemName });

		if (defaultValue !== undefined && defaultValue !== "") {
			for (var i = 0; i < optionList.length; i++) {
				if (optionList[i] === defaultValue) theItem.selection = i;
			}
		}

		if (required && theItem.selection === null) theItem.selection = 0;

		return theItem;
	},

	addGroup: function (menuObject, caption, name) {
		return menuObject.add("group");
	},

	addTabPanel: function (menuObject, itemName) {
		var tabPanel = menuObject.add("tabbedpanel");
		//tabPanel.preferredSize = [300, ""];
		return tabPanel;
	},

	addTab: function (menuObject, caption, itemName) {
		return menuObject.add("tab", undefined, caption, { name: itemName });
	},

	addPanel: function (menuObject, caption, itemName) {
		var thePanel = menuObject.add("panel", undefined, caption);
		//thePanel.preferredSize = [300, 800];
		return thePanel;
	},

	addTextfield: function (menuObject, caption, defaultValue, itemName) {
		if (caption !== "") this.addStaticText(menuObject, caption);
		return menuObject.add("edittext", undefined, defaultValue, { name: itemName });
	},

	addTreeView: function (menuObject, itemName) {
		return menuObject.add("treeview", [0, 0, 250, 450]);
	},

	addTreeViewNode: function (menuObject, caption, itemName) {
		return menuObject.add("node", caption, { name: itemName });
	},

	addTreeViewItem: function (menuObject, caption, itemName) {
		return menuObject.add("item", caption, { name: itemName });
	},

	addButton: function (menuObject, caption) {
		return menuObject.add("button", undefined, caption);
	},

	getSingleSelectedItem: function () {
		var selectableObject;
		var currentDocument = this.getCurrentDocument();

		EFI.Utils.addDebugLog("EFI.UIHelper.getSingleSelectedItem: selections - " + currentDocument.selection.length);

		if (currentDocument.selection.length !== 0) {
			var mySelection = currentDocument.selection;

			if (mySelection.length === 1) {
				mySelection = mySelection[0];

				EFI.Utils.addDebugLog("EFI.UIHelper.getSingleSelectedItem: selected - " + mySelection.constructor.name);
				if (EFI.Utils.hasProperty(mySelection, "id")) EFI.Utils.addDebugLog("EFI.UIHelper.getSingleSelectedItem: selected id - " + mySelection.id);

				selectableObject = EFI.ObjectFactory.getInstance("selectableObject", mySelection);
				selectableObject.hasSingleSelection = true;
			}
		}

		if (!selectableObject) selectableObject = EFI.ObjectFactory.getInstance("selectableObject");

		return selectableObject;
	},

	getCurrentDocument: function () {
		if (app.documents.length > 0) {
			return app.activeDocument;
		} else {
			return null;
		}
	},

	//returns document layers, but note it is top to bottom order
	getLayers: function () {
		var aryLayers = EFI.UIHelper.getCurrentDocument().layers;

		//NOT USING - could not get proper array to use reverse function, doing manually
		var aryReverseLayers = [];
		for (var i = aryLayers.length - 1; i >= 0; i--)
			aryReverseLayers.push(aryLayers[i]);

		return aryReverseLayers;
	},

	documentsOpenCount: function () {
		return app.documents.length;
	},

	checkDocumentLoading: function () {
		var documentsOpen = EFI.UIHelper.documentsOpenCount();
		var canOpenPlugin = false;

		if (documentsOpen === 0) {
			alert(EFI.I18n.noDocumentOpenError, EFI.I18n.alertErrorTitle, true);
		} else {
			canOpenPlugin = true;
		}

		return canOpenPlugin;
	},

	removeChildObjects: function (uiObject) {
		var children = uiObject.children;

		while (children[0]) {
			uiObject.remove(children[0]);
		}
	},

	refreshLayout: function (uiObject) {
		uiObject.layout.layout(true);
	},

	addButtonClickEvent: function (button, functionCall) {
		button.onClick = functionCall;
	},

	addDropDownChangeEvent: function (dropDown, functionCall) {
		dropDown.onChange = functionCall;
	},

	addTextChangeEvent: function (textField, functionCall) {
		textField.onChange = functionCall;
	},

	getVerticalAlign: function (textFrame) {
		var verticaAlign = "";

		switch (textFrame.textFramePreferences.verticalJustification) {
			case VerticalJustification.topAlign:
				verticaAlign = "top";
				break;
			case VerticalJustification.bottomAlign:
				verticaAlign = "bottom";
				break;
			case VerticalJustification.centerAlign:
				verticaAlign = "center";
				break;
			case VerticalJustification.justifyAlign:
				verticaAlign = "justify";
				break;
		}

		return verticaAlign;
	},

	getRotation: function (item) {
		var rotation = 0;

		if (item) {
			if (item.parentTextFrames) {
				if (item.parentTextFrames[0]) {
					rotation = item.parentTextFrames[0].rotationAngle;
				}
			}
		}

		return rotation;
	},

	getLayerId: function (item) {
		var layerId = "";

		if (item) {
			if (item.parentTextFrames) {
				if (item.parentTextFrames[0]) {
					layerId = item.parentTextFrames[0].itemLayer.id;
				}
			}
		}

		return layerId;
	},

	getXCoordinate: function (item) {
		var x = undefined;

		if (item) {
			var myDocument = EFI.UIHelper.getCurrentDocument();
			var origSetting = myDocument.viewPreferences.horizontalMeasurementUnits;

			myDocument.viewPreferences.horizontalMeasurementUnits = MeasurementUnits.POINTS;

			if (EFI.Utils.hasProperty(item, "horizontalOffset")) {
				x = item.horizontalOffset;
			} else if (EFI.Utils.hasProperty(item, "geometricBounds")) {
				x = item.geometricBounds[EFI.ConfigSettings.objectBounds.x1];
			}

			myDocument.viewPreferences.horizontalMeasurementUnits = origSetting;
		}

		return x;
	},

	getYCoordinate: function (item) {
		var y = undefined;

		if (item) {
			if (EFI.Utils.hasProperty(item, "baseline")) {
				y = item.baseline;
			} else if (EFI.Utils.hasProperty(item, "geometricBounds")) {
				y = item.geometricBounds[EFI.ConfigSettings.objectBounds.y1];
			}
		}

		return y;
	},

	getWidth: function (item) {
		var width = undefined;

		if (EFI.Utils.hasProperty(item, "geometricBounds")) {
			var bounds = item.geometricBounds;

			if (bounds) width = bounds[EFI.ConfigSettings.objectBounds.x2] - bounds[EFI.ConfigSettings.objectBounds.x1];
		}

		return width;
	},

	getHeight: function (item) {
		var bounds = item.geometricBounds;
		var height = undefined;

		if (bounds) height = bounds[EFI.ConfigSettings.objectBounds.y2] - bounds[EFI.ConfigSettings.objectBounds.y1];

		return height;
	},

	switchXToPoints: function () {
		var myDocument = EFI.UIHelper.getCurrentDocument();
		var origSetting = myDocument.viewPreferences.horizontalMeasurementUnits;

		myDocument.viewPreferences.horizontalMeasurementUnits = MeasurementUnits.POINTS;

		return origSetting;
	},

	switchXToOriginal: function (origSetting) {
		var myDocument = EFI.UIHelper.getCurrentDocument();
		myDocument.viewPreferences.horizontalMeasurementUnits = origSetting;
	},

	switchYToPoints: function () {
		var myDocument = EFI.UIHelper.getCurrentDocument();
		var origSetting = myDocument.viewPreferences.verticalMeasurementUnits;

		myDocument.viewPreferences.verticalMeasurementUnits = MeasurementUnits.POINTS;

		return origSetting;
	},

	switchYToOriginal: function (origSetting) {
		var myDocument = EFI.UIHelper.getCurrentDocument();
		myDocument.viewPreferences.verticalMeasurementUnits = origSetting;
	},

	getDocumentName: function () {
		var myDocument = EFI.UIHelper.getCurrentDocument();
		var name = myDocument.name;

		name = name.substring(0, name.lastIndexOf("."));

		return name;
	},

	getDocumentID: function () {
		var newGUID = EFI.Utils.getGUID();
		return newGUID;
	},

	//get Item By Id in its actual Type eg. TextFrame
	getItemById: function (id) {
		var fieldManager = EFI.ObjectFactory.getInstance("fieldManager");
		var pageItem;

		var allFrames = EFI.Utils.toArray(EFI.UIHelper.getCurrentDocument().allPageItems);

		pageItems = EFI.Utils.selectWhere(id, "id", allFrames);
		if (pageItems && pageItems.length > 0) {
			pageItem = pageItems[0];

			return pageItem;
		} else {
			return null;
		}
	},


	createPalette: function (title) {
		var ui = {};
		ui.window = new Window('palette', title);
		return ui;
	},

    // show a progressbar
    progress: function(steps) {
        var progressBar;
        var progressMessage;
        var window;
        var maxValue = steps;
        
        window = new Window("palette", "Progress", undefined, {closeButton: false});

        progressMessage = window.add("statictext");

        progressMessage.preferredSize = [450, -1]; // 450 pixels wide, default height.

        if (steps) {
            progressBar = window.add("progressbar", undefined, 0, steps);
            progressBar.preferredSize = [450, -1]; // 450 pixels wide, default height.
        }

        show = function(){
            window.show();
        };
    
        close = function(){
            window.close();
        };
    
        incrementAndSetMessage = function(message){
            if(progressBar.value >= maxValue) {
                progressBar.value = 0;
             } else {
                progressBar.value++;
            }

            progressMessage.text = message;
            window.update();
        };
    
        increment = function () {
            if(progressBar.value >= maxValue) {
                progressBar.value = 0;
             } else {
                progressBar.value++;
            }
            window.update();
        };

        setProgressMessage = function (message) {
            progressMessage.text = message;
            window.update();
        };

        return { 
            show: show,
            close: close, 
            setProgressMessage: setProgressMessage, 
            increment: increment,
            incrementAndSetMessage: incrementAndSetMessage
         };

    }
};

//-----------------//UIHelper---------------