﻿
//----------------Utils-------------
//Helper functions
EFI.Utils = {
    getObjectOwnPropertiesNameOnly: function (f) {
        var props = f.reflect.properties;
        var array = [];

        for (var i = 0; i < props.length; i++)
            try {
                if (f.hasOwnProperty(props[i].name)) array.push(props[i].name);
            } catch (e) {
                EFI.Utils.addDebugLog("EFI.Utils.getObjectOwnPropertiesNameOnly: " + e);
            }

        return array;
    },

    getObjectProperties: function (f) {
        var props = f.reflect.properties;
        var array = [];

        for (var i = 0; i < props.length; i++)
            if (props[i].name) {
                try {
                    array.push(props[i].name + ": " + f[props[i].name]);
                } catch (e) {
                    EFI.Utils.addDebugLog("EFI.Utils.getObjectProperties: " + e);
                }
                array.sort();
            }

        return array;
    },

    //Does a popup of all Properties an object has
    displayObjectProperties: function (f) {
        alert(f.reflect.name);
        var array = EFI.Utils.getObjectProperties(f);
        alert(array.join("\r"));
    },

    //Does a popup of all the Methods an object hs
    getObjectMethods: function (m) {
        var props = m.reflect.methods.sort();
        var allProps = "";

        allProps += "\rMethods";
        for (var i = 0; i < props.length; i++)
            allProps += props[i].name + "\r";
        EFI.Utils.addDebugLog("EFI.Utils.getObjectMethods - " + allProps);
    },

    //Load a javascript object from an InDesign object
    getObjectState: function (objectFrom, saveLabel) {
        var theObject = undefined;

        if (objectFrom.extractLabel(saveLabel) !== "") theObject = EFI.Utils.getObjectFromJSON(objectFrom.extractLabel(saveLabel));

        return theObject;
    },

    //Save a javascript object to an InDesign object
    saveObjectState: function (saveTo, objectToSave, saveLabel) {
        if (saveTo !== undefined && objectToSave !== undefined) {
            EFI.Utils.addDebugLog("EFI.Utils.saveObjectState - " + objectToSave.constructor.name);
            saveTo.insertLabel(saveLabel, EFI.Utils.getJSON(objectToSave));
        }
    },

    addDebugLog: function (message) {
        // if you uncomment next line, then Extend Script Toolkit will open on customers computer if ESTK is installed
        //$.writeln(message); 
    },

    isTextObject: function (theObject) {
        return EFI.Utils.hasProperty(theObject, "parentTextFrames");
    },

    //params of type File
    zipFolder: function (source, destination) {
        EFI.Utils.addDebugLog("EFI.Utils.zipFolder - from " + source.fullName + " to " + destination.fullName);
        app.packageUCF(source, destination);
    },

    //params of type File
    unZipFile: function (source, destination) {
        EFI.Utils.addDebugLog("EFI.Utils.unZipFile - from " + source.fullName + " to " + destination.fullName);
        app.unpackageUCF(source, destination);
    },

    hasProperty: function (theObject, propertyName) {
        return theObject.hasOwnProperty(propertyName);
    },

    getJSON: function (theObject) {
        var JSONString = "";

        if (theObject) JSONString = JSON.stringify(theObject);

        return JSONString;
    },

    getObjectFromJSON: function (theObject) {
        var returnObject = undefined;

        returnObject = JSON.parse(theObject);

        return returnObject;
    },

    //param: fileToWrite of type File
    writeFile: function (fileToWrite, writeString) {
        EFI.Utils.addDebugLog("EFI.Utils.writeFile - try save file to " + fileToWrite.fullName);

        var fileOpened = fileToWrite.open("w");
        EFI.Utils.addDebugLog("EFI.Utils.writeFile - opened " + fileOpened);

        var fileWritten = false;
        if (fileOpened) {
            fileWritten = fileToWrite.write(writeString);
            EFI.Utils.addDebugLog("EFI.Utils.writeFile - file written " + fileWritten);
        }

        var fileClosed = true;
        if (fileOpened) fileClosed = fileToWrite.close();

        EFI.Utils.addDebugLog("EFI.Utils.writeFile - file closed " + fileClosed);

        return fileWritten;
    },

    //param: fileSource and fileDestination - AK
    copyFile: function (fileSource, fileDestinationFolder) {
        var sourcePath = File(fileSource);
        var myFileName = sourcePath.name.substr(0, sourcePath.name.lastIndexOf("."));
        var myFileExt = sourcePath.name.substr(sourcePath.name.lastIndexOf(".") + 1, sourcePath.name.length);
        var targetPath = File(fileDestinationFolder + "/" + myFileName + "." + myFileExt);
        sourcePath.copy(targetPath);
    },

    //param: fileToDelete of type File
    deleteFile: function (fileToDelete) {
        EFI.Utils.addDebugLog("EFI.Utils.deleteFile: " + fileToDelete.fullName);

        return fileToDelete.remove();
    },

    //param: folderToDelete of type Folder
    deleteFolder: function (folderToDelete) {
        var aryItems = folderToDelete.getFiles();
        EFI.Utils.addDebugLog("EFI.Utils.deleteFolder: " + folderToDelete.fullName + " number items: " + aryItems.length);

        for (var i = 0; i < aryItems.length; i++) {
            var itemToDelete = aryItems[i];

            if (itemToDelete instanceof File) {
                EFI.Utils.deleteFile(itemToDelete);
            } else if (itemToDelete instanceof Folder) {
                EFI.Utils.deleteFolder(itemToDelete);
            }
        }

        return folderToDelete.remove();
    },

    //param: folderToCreate of type Folder
    createCheckFolder: function (folderToCreate) {
        if (!folderToCreate.exists)
            folderToCreate.create();

        return folderToCreate.fullName;
    },

    getPDFExportPreferences: function () {
        var pdfPresets = [];

        for (var i = 0; i < app.pdfExportPresets.length; i++) {
            pdfPresets.push(String(app.pdfExportPresets.item(i).name));
        }

        return pdfPresets;
    },

    getDocumentWidth: function () {
        var curDoc = EFI.UIHelper.getCurrentDocument();

        var xOrig = EFI.UIHelper.switchXToPoints();
        var width = curDoc.documentPreferences.pageWidth;
        EFI.UIHelper.switchXToOriginal(xOrig);

        return width;
    },

    getDocumentHeight: function () {
        var curDoc = EFI.UIHelper.getCurrentDocument();

        var yOrig = EFI.UIHelper.switchYToPoints();
        var height = curDoc.documentPreferences.pageHeight;
        EFI.UIHelper.switchYToOriginal(yOrig);

        return height;
    },

    getBleedTop: function () {
        var curDoc = EFI.UIHelper.getCurrentDocument();

        var yOrig = EFI.UIHelper.switchYToPoints();
        var bleed = EFI.Utils.getBleedUniformSize();
        if (bleed === -1) bleed = curDoc.documentPreferences.documentBleedTopOffset;
        EFI.UIHelper.switchYToOriginal(yOrig);

        return bleed;
    },

    getBleedBottom: function () {
        var curDoc = EFI.UIHelper.getCurrentDocument();

        var yOrig = EFI.UIHelper.switchYToPoints();
        var bleed = EFI.Utils.getBleedUniformSize();
        if (bleed === -1) bleed = curDoc.documentPreferences.documentBleedBottomOffset;
        EFI.UIHelper.switchYToOriginal(yOrig);

        return bleed;
    },

    getBleedLeft: function () {
        var curDoc = EFI.UIHelper.getCurrentDocument();

        var xOrig = EFI.UIHelper.switchXToPoints();
        var bleed = EFI.Utils.getBleedUniformSize();
        if (bleed === -1) bleed = curDoc.documentPreferences.documentBleedInsideOrLeftOffset;
        EFI.UIHelper.switchXToOriginal(xOrig);

        return bleed;
    },

    getBleedRight: function () {
        var curDoc = EFI.UIHelper.getCurrentDocument();

        var xOrig = EFI.UIHelper.switchXToPoints();
        var bleed = EFI.Utils.getBleedUniformSize();
        if (bleed === -1) bleed = curDoc.documentPreferences.documentBleedOutsideOrRightOffset;
        EFI.UIHelper.switchXToOriginal(xOrig);

        return bleed;
    },

    getBleedUniformSize: function () {
        var curDoc = EFI.UIHelper.getCurrentDocument();

        if (curDoc.documentPreferences.documentBleedUniformSize) {
            return curDoc.documentPreferences.documentBleedTopOffset;
        } else {
            return -1;
        }
    },

    getParentTextFrame: function (item) {
        var textFrame = undefined;

        if (item) {
            if (item.parentTextFrames) {
                if (item.parentTextFrames[0]) {
                    textFrame = item.parentTextFrames[0];
                }
            }
        }

        return textFrame;
    },

    //copy properties from the fromObject over to the toObject. Used for re-initializing objects to include any new variables
    copyObjectFieldValueProperties: function (fromObject, toObject) {
        if (fromObject) {
            var aryProperties = EFI.Utils.getObjectOwnPropertiesNameOnly(fromObject);

            for (var i = 0; i < aryProperties.length; i++) {
                if (fromObject[aryProperties[i]] && fromObject[aryProperties[i]].fieldValue && toObject[aryProperties[i]]) toObject[aryProperties[i]].fieldValue = fromObject[aryProperties[i]].fieldValue;
            }
        }
    },

    //clean object to only have property values defined from .fieldValue definition
    cleanObjectProperties: function (cleanObject) {
        var newObject = {};

        if (cleanObject) {
            var aryProperties = EFI.Utils.getObjectOwnPropertiesNameOnly(cleanObject);

            for (var i = 0; i < aryProperties.length; i++) {
                if (cleanObject[aryProperties[i]] && cleanObject[aryProperties[i]].fieldValue) newObject[aryProperties[i]] = cleanObject[aryProperties[i]].fieldValue;
            }
        }

        return newObject;
    },

    //read a text file
    //param: fileToRead of type File
    readTextFile: function (fileToRead) {
        var contents = "";

        if (fileToRead.exists) {
            fileToRead.open("r");
            contents = fileToRead.read();
        }

        return contents;
    },

    //add character set to textfield
    //param: textfield of type InDesign textField
    addCharacterSet: function (textField) {
        var characters = "!\"#$%&'()*+,-./ 0123456789:;<=>?@ ABCDEFGHIJKLM NOPQRSTUVWXYZ \\][^_` abcdefghijklm nopqrstuvwxyz{|}~";

        textField.contents = characters;
    },

    getCurrentFolder: function () {
        return new File((new File($.fileName)).parent.fullName);
    },

    getTempFolder: function () {
        return new File(Folder.temp + "/EFI/InDesignPlugin/tempExport");
        //return new File(EFI.Utils.getCurrentFolder() + "/tempExport");
    },

    getFontsFolder: function () {
        return new Folder(Folder.startup + "/Resources/Required/Fonts/");
    },


    //select value/key from array
    selectWhere: function (value, key, array) {
        var i = array.length;
        var t;
        var filtered = [];

        while (i--) {
            t = array[i];

            if (t && t[key] === value) {
                filtered.push(t);
            }
        }
        return filtered;
    },

    //convert objects to array
    toArray: function (objects) {
        var i = objects.length;
        var array = [];

        while (i--) {
            array.push(objects[i]);
        }

        return array;
    },

    //InDesign screen dpi to 72dpi measurement
    convertScreenDPI: function (input) {
        return input * 0.75;
    },

    getParameters: function (object) {
        var aryProperties = EFI.Utils.getObjectProperties(object);
        var strProperties = "";

        for (var i = 0; i < aryProperties.length; i++) {
            strProperties += aryProperties[i] + '\n';
        }
        this.debug(strProperties);
    },

    debug: function (strInfo) {
        var outfile = File('~/Desktop/Debug.txt');
        outfile.encoding = 'UTF-8';
        outfile.open('w');
        outfile.write(strInfo);
        outfile.close();
        outfile.execute();
    },

    getGUID: function () {
        function partID() {
            return Math.floor((1 + Math.random()) * 0x10000)
                .toString(16)
                .substring(1);
        }
        return partID() + partID() + '-' + partID() + '-' + partID() + '-' +
            partID() + '-' + partID() + partID() + partID();
    },

    escapeRegExp: function (str) {
        return str.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, "\\$&");
    },

    getFrameCoOrdinates: function (objFrame) {
        var refPoint = this.getReferencePoint();
        var curRotation = objFrame.baseObject.rotationAngle;
        var frmDetails = [];

        if (curRotation !== 0) {
            this.setReferencePoint(EFI.ConfigSettings.anchorPoint.CENTER_ANCHOR);
            objFrame.baseObject.rotationAngle = 0;
            frmDetails = this.getFrameDetails(objFrame);
            objFrame.baseObject.rotationAngle = curRotation;
            this.setReferencePoint(refPoint);
            if (curRotation > 0) {
                frmDetails.Rotation = 360 - curRotation;
            } else {
                frmDetails.Rotation = - 1 * curRotation;
            }

        } else {
            frmDetails = this.getFrameDetails(objFrame);
            frmDetails.Rotation = curRotation;
        }
        return frmDetails;
    },

    getFrameDetails: function (objFrame) {
        var frameInfo = [];
        var objImgField = new EFI.ImageField(objFrame).baseObject;
        var strStrokeWeight = "0";

        frameInfo.FileName = objFrame.getImageObject().placeHolder;
        frameInfo.Left = this.roundNumber(EFI.UIHelper.getXCoordinate(objFrame.baseObject), 4);
        frameInfo.Top = this.roundNumber(EFI.UIHelper.getYCoordinate(objFrame.baseObject), 4);
        frameInfo.Width = this.roundNumber(EFI.UIHelper.getWidth(objFrame.baseObject), 4);
        frameInfo.Height = this.roundNumber(EFI.UIHelper.getHeight(objFrame.baseObject), 4);

        frameInfo.Rotation = 0;

        frameInfo.PadLeft = this.roundNumber(parseFloat(objImgField.getInternalGraphicX()), 4);
        frameInfo.PadTop = this.roundNumber(parseFloat(objImgField.getInternalGraphicY()), 4);
        frameInfo.PadRight = this.roundNumber(parseFloat(frameInfo.Width) - (parseFloat(frameInfo.PadLeft) + parseFloat(objImgField.getInternalScaledImageWidth())), 4);
        frameInfo.PadBottom = this.roundNumber(parseFloat(frameInfo.Height) - (parseFloat(frameInfo.PadTop) + parseFloat(objImgField.getInternalScaledImageHeight())), 4);

        frameInfo.Opacity = this.roundNumber(parseFloat(objFrame.baseObject.transparencySettings.blendingSettings.properties.opacity) / 100, 4);

        strStrokeWeight = objFrame.baseObject.strokeWeight;
        if (strStrokeWeight !== 0) frameInfo.BorderColor = this.getSwatchOrColor(objFrame.baseObject.strokeColor);
        frameInfo.BorderThickness = strStrokeWeight;

        if (objFrame.baseObject.topLeftCornerOption === EFI.ConfigSettings.CornerOption.NONE) {
            frameInfo.CornerRadius = "0";
        } else {
            if (objFrame.baseObject.topLeftCornerOption === EFI.ConfigSettings.CornerOption.ROUNDED_CORNER) {
                frameInfo.CornerRadius = objFrame.baseObject.topRightCornerRadius;
            } else {
                frameInfo.CornerRadius = "0";
                alert(EFI.I18n.nonRoundedCornersVarImagesError, EFI.I18n.alertErrorTitle, true);
            }
        }
        //topLeftCornerRadius bottomLeftCornerRadius bottomRightCornerRadius

        return frameInfo;
    },

    // ToDo: handle gradients (currently not supported in SmartCanvas
    getSwatchOrColor: function (objColor) {

        if (objColor && objColor.hasOwnProperty("name")) {
            var clrName = objColor.name;
            var model = this.getColorModelString(objColor);

            if (clrName === "Black") {
                clrName = "[Black]";
            } else if (clrName === "Paper") {
                clrName = "[Paper]";
            }

            if (clrName && clrName.toLowerCase() !== "none") {

                if (model === "spot") {
                    return "swatch," + clrName + " 100%";
                } else {
                    return "swatch," + clrName;
                }
            }
        }

        return this.getColor(objColor);
    },

    getSwatchNameOrColor: function (objColor) {

        if (objColor && objColor.hasOwnProperty("name")) {
            var clrName = objColor.name;
            var model = this.getColorModelString(objColor);

            if (clrName === "Black") {
                clrName = "[Black]";
            } else if (clrName === "Paper") {
                clrName = "[Paper]";
            }

            if (clrName && clrName.toLowerCase() !== "none") {

                if (model === "spot") {
                    return "swatch, " + clrName + " 100%";
                } else {
                    return "swatch, " + clrName;
                }
            }
        }

        return this.getColor(objColor);
    },

    // ToDo: handle gradients (currently not supported in SmartCanvas
    getColor: function (objColor) {
        var clrSpace = EFI.ConfigSettings.colorSpace;
        var clrValue = "";

        if (objColor && objColor.hasOwnProperty("space")) {
            switch (objColor.space) {
                case clrSpace.CMYK:
                    clrValue = "cmyk";
                    break;
                case clrSpace.LAB:
                    clrValue = "lab";
                    break;
                case clrSpace.MIXEDINK:
                    clrValue = "mixedink";
                    break;
                case clrSpace.RGB:
                    clrValue = "rgb";
                    break;
                default:
                //do nothing
            }

            if (clrValue !== "") {
                clrValue = clrValue + "," + objColor.colorValue.toString();
            }
        }

        return clrValue;
    },


    getColorSpaceString: function (objColor) {
        var clrSpace = EFI.ConfigSettings.colorSpace;
        var clrValue = "";

        if (objColor && objColor.hasOwnProperty("space")) {
            switch (objColor.space) {
                case clrSpace.CMYK:
                    clrValue = "cmyk";
                    break;
                case clrSpace.LAB:
                    clrValue = "lab";
                    break;
                case clrSpace.MIXEDINK:
                    clrValue = "mixedink";
                    break;
                case clrSpace.RGB:
                    clrValue = "rgb";
                    break;
                default:
                //do nothing
            }
        }

        return clrValue;
    },

    getColorModelString: function (objColor) {

        if (objColor && objColor.hasOwnProperty("model")) {

            if (objColor.model == ColorModel.PROCESS) {
                return "process";
            }

            return "spot";
        }

        return "";
    },



    getReferencePoint: function () {
        return app.activeWindow.transformReferencePoint;
    },

    setReferencePoint: function (refPoint) {
        app.activeWindow.transformReferencePoint = refPoint;
    },

    roundNumber: function (strVal, intPlaces) {
        return parseFloat(strVal).toFixed(intPlaces);
    },

    getSmartCanvasLengthUnitFromMeasurementUnit: function (mUnit) {

        switch (mUnit) {
            case MeasurementUnits.POINTS:
                return 0;

            case MeasurementUnits.PICAS:
                return 1;

            case MeasurementUnits.INCHES:
                return 2;

            case MeasurementUnits.CICEROS:
                return 3;

            case MeasurementUnits.MILLIMETERS:
                return 4;

            case MeasurementUnits.CENTIMETERS:
                return 5;

            default:
                return 0;
        }

        return 0;
    },

    getSmartCanvasReferencePoint: function (transformReferencePoint) {
        switch (transformReferencePoint) {

            case AnchorPoint.TOP_LEFT_ANCHOR:
                return "ul";

            case AnchorPoint.TOP_CENTER_ANCHOR:
                return "um";

            case AnchorPoint.TOP_RIGHT_ANCHOR:
                return "ur";

            case AnchorPoint.LEFT_CENTER_ANCHOR:
                return "cl";

            case AnchorPoint.CENTER_ANCHOR:
                return "cc";

            case AnchorPoint.RIGHT_CENTER_ANCHOR:
                return "cr";

            case AnchorPoint.BOTTOM_LEFT_ANCHOR:
                return "ll";

            case AnchorPoint.BOTTOM_CENTER_ANCHOR:
                return "lm";

            case AnchorPoint.BOTTOM_RIGHT_ANCHOR:
                return "lr";


            default:
                return "ul";
        }
    },

    getTextAlign: function (justification) {

        switch (justification) {

            case Justification.LEFT_ALIGN:
                return "left";

            case Justification.RIGHT_ALIGN:
                return "right";

            case Justification.CENTER_ALIGN:
                return "center";

            case Justification.LEFT_JUSTIFIED:
                return "justify";

            case Justification.CENTER_JUSTIFIED:
                return "justify";

            case Justification.FULLY_JUSTIFIED:
                return "last";

            default:
                return "left";
        }

        return "left";
    },

    getFontStyle: function (style) {

        switch (style) {

            case "Regular":
                return "normal";

            default:
                return "normal";
        }

        return "normal";
    },

    getFontWeight: function (weight) {

        if (weight && typeof weight === "string") {

            const _weight = weight.toLowerCase();

            switch (_weight) {

                case "thin":
                    return "100";

                case "extra light":
                    return "200";

                case "light":
                    return "300";

                case "normal":
                case "regular":
                case "book":
                    return "400";

                case "medium":
                    return "500";

                case "demi":
                case "demi bold":
                case "semi bold":
                case "demi-bold":
                case "semi-bold":
                    return "600";

                case "bold":
                    return "700";

                case "heavy":
                    return "800";

                case "black":
                    return "900";

                default:
                    return "400";
            }
        }

        return "400";
    },

    htmlEncode: function (value) {
        return String(value)
            .replace(/&/g, '&amp;')
            .replace(/"/g, '&quot;')
            .replace(/'/g, '&apos;')
            .replace(/</g, '&lt;')
            .replace(/>/g, '&gt;');
    },

    htmlDecode: function (value) {
        return String(value)
            .replace(/&quot;/g, '"')
            .replace(/&#39;/g, "'")
            .replace(/&apos;/g, "'")
            .replace(/&lt;/g, '<')
            .replace(/&gt;/g, '>')
            .replace(/&amp;/g, '&');
    },

    isVariableObject: function (selId) {
        var currentDocument = EFI.UIHelper.getCurrentDocument();
        var boxObject = "";
        var blnReturn = false;
        var strFieldValue = "";
        var labelName = "EFISmartCanvasExporter";

        if (currentDocument.extractLabel(labelName) !== "") {
            boxObject = JSON.parse(currentDocument.extractLabel(labelName));
            for (var i = 0; i < boxObject.boxProperties.length; i++) {
                if (boxObject.boxProperties[i].id === selId) {
                    strFieldValue = boxObject.boxProperties[i].boxVariable.fieldValue;
                    if (strFieldValue === false) {
                        blnReturn = false;
                    } else {
                        blnReturn = true;
                    }
                }
            }
        }
        return blnReturn;
    },

    hasVariableItemsInGroups: function () {

        var allFrames = app.activeDocument.allPageItems;

        for (i = 0; i < allFrames.length; i++) {
            var frame = allFrames[i];
            if (frame.pageItems.length > 1) {

                for (var pi = 0; pi < frame.pageItems.length; pi++) {
                    var pageItem = frame.pageItems[pi];

                    if (EFI.Utils.isVariableObject(pageItem.id)) {
                        // variable item
                        alert('Document contains variable images or text inside groups, which is not supported.\rPlease ungroup those frames. Export aborted!\r', "Error: Invalid grouping", true);
                        return true;
                    }

                }
            }

        }

        return false;
    },

    hasMissingFonts: function () {

        var usedFonts = app.activeDocument.fonts;
        var missingFonts = [];

        for (var i = 0; i < usedFonts.length; i++) {
            var usedFont = usedFonts[i];
            if (usedFont.status == FontStatus.NOT_AVAILABLE || usedFont.status == FontStatus.SUBSTITUTED) {
                missingFonts.push(usedFont.name);
            }
        }

        if (missingFonts.length != 0) {
            alert('Document contains missing fonts. Export aborted!\r' + 'Unavailable fonts:\r' + missingFonts.join('\r'), "Error: Missing fonts", true);
            return true;
        }

        return false;
    },


    hasMissingImages: function () {

        var usedGraphics = app.activeDocument.allGraphics;
        var missingGraphics = [];

        for (var i = 0; i < usedGraphics.length; i++) {
            var graphic = usedGraphics[i];
            var graphicLink = graphic.itemLink;

            if (graphicLink && graphicLink.status === LinkStatus.LINK_MISSING) {
                missingGraphics.push(graphicLink.filePath);
            }
        }

        if (missingGraphics.length != 0) {
            alert('Document contains missing image links. Export aborted!\r' + 'Unavailable images:\r' + missingGraphics.join('\r'), "Error: Missing images", true);
            return true;
        }

        return false;
    },

    hasTextAlignJustifyCenter: function () {
        // check if text alignment "justifycenter" was used, which is not supported by SmartCanvas
        var doc = app.activeDocument;
        var allTextFrames = [];

        for (var i = doc.allPageItems.length - 1; i >= 0; i--) {
            var pi = doc.allPageItems[i];

            if (pi instanceof TextFrame || pi instanceof TextPath) {

                for (var pg = 0; pg < pi.paragraphs.length; pg++) {
                    var paragraph = pi.paragraphs[pg];

                    if (paragraph.justification === Justification.CENTER_JUSTIFIED) {
                        return true;
                    }
                }

            }

        }

        return false;
    },

    hasQRCodes: function () {
        // check if document contains QR Codes
        var doc = app.activeDocument;
        var allTextFrames = [];

        for (var i = doc.allPageItems.length - 1; i >= 0; i--) {
            var pi = doc.allPageItems[i];

            if (pi instanceof EPS && pi.itemLink && (EFI.Utils.isVariableObject(pi.id) || EFI.Utils.isVariableObject(pi.parent.id))) {
                var itemLink = pi.itemLink;
                if (itemLink.name.indexOf("QR Code") === 0) {
                    return true;
                }
            }

        }

        return false;
    },

    hasImageFileExtension: function (fileName) {

        if (fileName && fileName !== "" && typeof fileName === "string") {
            return fileName.match(/.(jpg|jpeg|png|apng|bmp|ico|webp|gif|tif|gif|eps|svg)$/i);
        }

        return false;
    },

    hasFontFileExtension: function (fileName) {

        if (fileName && fileName !== "" && typeof fileName === "string") {
            return fileName.match(/.(otf|ttf|woff|woff2|fnt|pfa|fot)$/i);
        }

        return false;
    },

    // we try to get the correct font from the Adobe Resource font folder
    getAdobeResourceFontLocations: function (fontFile) {

        // Adobe InDesign Required Fonts folder
        var fontsFolder = EFI.Utils.getFontsFolder();

        var locations = [];

        try {

            if (fontsFolder && fontsFolder.exists) {
                var fonts = fontsFolder.getFiles();
                for (var i = 0; i < fonts.length; i++) {
                    var folderFontFile = fonts[i];
                    var fontFileFullName = folderFontFile.fullName.toLowerCase();

                    if (fontFileFullName.indexOf(fontFile.postscriptName.toLowerCase()) !== -1
                        || fontFileFullName.indexOf(fontFile.name.toLowerCase()) !== -1
                        || fontFileFullName.indexOf(fontFile.fullName.toLowerCase()) !== -1
                    ) {

                        locations.push(folderFontFile.fullName);
                    }
                }
            }
        } catch (e) {
            EFI.Utils.addDebugLog("error in getAdobeResourceFontLocations: " + e);
        }

        return locations;
    },

    arrayContains: function (searchArray, searchValue) {
        if (searchValue && searchArray && searchArray.length > 0) {
            for (var i = 0; i < searchArray.length; i++) {
                if (searchArray[i] === searchValue) {
                    return true;
                }
            }
        }

        return false;
    },

    // ID without layer information
    getPlainId: function (id) {

        var stringId = id + ""; // sometimes id comes in as a number

        var stringParts = stringId.split("-");

        if (stringParts.length === 2) {
            return stringParts[0];
        }

        return stringId;
    },

    isEven: function (value) {
        if (value % 2 == 0)
            return true;
        else
            return false;
    },

    getOS: function () {
        try {
            return $.os.toLowerCase().indexOf("mac") !== -1 ? "mac" : "windows";
        } catch (error) {
            return "";
        }
    }


};
//----------------//Utils-----------