#include "_LoadAll.jsx";

//-----------------Main Code------------

if (EFI.UIHelper.checkDocumentLoading()) {
    
        var exportResultMessage = "";
        var usesJustifyCenter = false;
        var hasQRCodes = false;
        
    try {
        var progressBar = EFI.UIHelper.progress(20);
        progressBar.show();
        
        progressBar.incrementAndSetMessage("Starting export...");

        // creates the temp folder if its not there
        var tempFolder = EFI.Utils.createCheckFolder(new Folder(EFI.Utils.getTempFolder() + "/randomFolder"));

        progressBar.incrementAndSetMessage("Inspecting text boxes.");
        usesJustifyCenter = EFI.Utils.hasTextAlignJustifyCenter();
        
        progressBar.incrementAndSetMessage("Search for QR Codes.");
        hasQRCodes = EFI.Utils.hasQRCodes();
        
        // check for correct facing pages settings
        var document = EFI.UIHelper.getCurrentDocument();
        var hasFacingPages = document.documentPreferences.facingPages;        
        var pagesPerDocument = document.documentPreferences.pagesPerDocument;
        
        if(hasFacingPages && pagesPerDocument > 2 && !EFI.Utils.isEven(pagesPerDocument)){
                progressBar.close();
                alert('Your document has facing pages, but your number of pages is odd. This combination is not possible in EFI SmartCanvas.\rPlease add or remove one page, so that it can be exported for EFI SmartCanvas. Export aborted!\r', "Error: Facing pages with odd number of pages", true);                
                exit();
        }
    
        progressBar.incrementAndSetMessage("Searching for missing images.");
        var hasMissingImages = EFI.Utils.hasMissingImages();
        if(hasMissingImages){
                progressBar.close();
                exit();
        }
        
        progressBar.incrementAndSetMessage("Searching for missing fonts.");        
        var hasMissingFonts = EFI.Utils.hasMissingFonts(progressBar);
        if(hasMissingFonts){
                progressBar.close();
                exit();
        }
    
        progressBar.incrementAndSetMessage("Searching for variable items in groups.");        
        var hasVariableItemsInGroups = EFI.Utils.hasVariableItemsInGroups();
        if(hasVariableItemsInGroups){
                progressBar.close();
                exit();
        }
        
        progressBar.incrementAndSetMessage("Waiting for user input.");
       
        EFI.ObjectFactory.getInstance("fieldManager").loadInitialSavedData();
        var myFile = File.saveDialog("Export", "*.zip");
    
        progressBar.incrementAndSetMessage("Processing...");
              
        if (myFile) {

            var override = true;
            
            var fullFileName = myFile.absoluteURI;

            if(fullFileName.toLowerCase().indexOf(".zip") === -1 || !fullFileName.toLowerCase().indexOf(".zip") === fullFileName.length - 4){
                // on MacOS we need to add this file extension
                fullFileName += ".zip";
            }

            var exportFile = File(fullFileName);
            
            if(exportFile.exists && EFI.Utils.getOS() === "windows"){
                override = confirm("File " + exportFile.name + " already exists! Do you want to override it?");
            }
            
            if(override){
                
                var exporter = new EFI.Exporter(exportFile.fullName, function(progressMessage){
                        progressBar.incrementAndSetMessage(progressMessage);
                 });
                
                var result = exporter.exportPackage(progressBar);
                
                progressBar.close();
                
                if(result && !result.success && result.errorMessage){
                    alert(result.errorMessage);
                } else {
                    exportResultMessage = EFI.I18n.fileExport.exportComplete;
                    if(usesJustifyCenter){
                        exportResultMessage += "\rFound unsupported text align 'JustifyCenter' in one or more text boxes. \rThe export uses now 'JustifyLeft' instead.\rOnly 'JustifyLeft' and 'JustifyFull' are supported by EFI SmartCanvas.\rYou can use the export file, but the import will maybe look different.";
                    }
                    if(hasQRCodes){
                        exportResultMessage += "\rFound variable QR Codes on the document, those are currently not supported by the exporter. \rYou can use the export file, but the import will not contain variable QR Codes";
                    }
                
                    alert(exportResultMessage, EFI.I18n.fileExport.exportCompleteDialogTitle);
                }
            }
        
        
        }

         progressBar.close();
         
    } catch(error){
            progressBar.close();
            alert (error, "Error", true);
    }
}

//-----------------End Main Code------------