﻿/************************************************************************/
/*  Function: EFISmartCanvasExporter									*/
/*  Purpose: Add Smart Canvas Exporter Context Menu						*/
/*  Author: Abhinav Kushalnagar											*/
/*  Created Date: 06 December 2016										*/
/*  Version: 1.0														*/
/*																		*/
/*      Installation:                                                   */
/*                                                                      */
/*      1) Place the current file into Scripts/Startup Scripts/         */
/*         (if the folder Startup Scripts doesn't exist, create it)     */
/*                                                                      */
/*                                                                      */
/************************************************************************/
#targetengine "EFISmartCanvasContextMenu";
#include "json2.js";

contextMenuInstaller();

function contextMenuInstaller(){
	var labelName =  "EFISmartCanvasExporter";
	var myLayoutContextMenu = app.menus.item("$ID/RtMouseLayout");
	var myBeforeDisplayListener = myLayoutContextMenu.addEventListener("beforeDisplay", myBeforeDisplayHandler, false);
	
	function myBeforeDisplayHandler(myEvent){
		
		var strTextSelected = "Static";
		var strLastTextSelected = "";
		
		var strImageSelected = "Static";
		var strLastImageSelected = "";

		if(app.documents.length != 0){
			if(app.selection.length >= 1){
				RemoveAllContextMenu();
				for(var i=0; i<app.selection.length; i++){
					
					var strSelectedId = app.selection[i].id;
					
					switch(app.selection[i].constructor.name){
						case "Rectangle":						
							strImageSelected = GetSelectionStatus(strSelectedId);
							if(strLastImageSelected !== "" && strImageSelected !== strLastImageSelected){
								strImageSelected = "Both"
							}							
							strLastImageSelected = strImageSelected;
							break;
						case "Oval":
						case "Polygon":
						case "GraphicLine":
						case "TextFrame":
							strTextSelected = GetSelectionStatus(strSelectedId);
							if(strLastTextSelected !== "" && strTextSelected !== strLastTextSelected){
								strTextSelected = "Both"
							}							
							strLastTextSelected = strTextSelected;
							break;
						default:
					}
				}
				
				if(strLastImageSelected !== ""){
					BuildContextMenu("Image", strLastImageSelected);
				}
				
				if(strLastTextSelected !== ""){
					BuildContextMenu("Text", strLastTextSelected);
				}
			}
		}
	}
	
	function myCheckForScriptMenuItem(myString){
		var myResult = false;
		try{
			var myScriptMenuAction = app.scriptMenuActions.item(myString);
			myScriptMenuAction.name;
			myResult = true
		}
		catch(myError){}
		//alert("Script menu action found? " + myString + "\n" + myResult);
		return myResult;  
	}
	
	function BuildContextMenu(strWhat, strSelected) {
		
		try {
			//if(myCheckForScriptMenuItem("EFI SmartCanvas") == false){
			//	var myMenuActionH = app.scriptMenuActions.add("EFI SmartCanvas");
			//	var myMenuHeader = myMenuActionH.enabled = false;
			//}
			//alert(strWhat + " - " + strSelected);
			
			if(myCheckForScriptMenuItem("Static " + strWhat) == true) app.scriptMenuActions.item("Static " + strWhat).remove();
			var myMenuActionS = app.scriptMenuActions.add("Static " + strWhat);			
			myMenuActionS.checked = strSelected === "Static" || strSelected === "Both";
			var myEventListenerS = myMenuActionS.eventListeners.add("onInvoke", myStaticEventHandler, false);

			if(myCheckForScriptMenuItem("Variable " + strWhat) == true) app.scriptMenuActions.item("Variable " + strWhat).remove();
			var myMenuActionV = app.scriptMenuActions.add("Variable " + strWhat);
			myMenuActionV.checked = strSelected === "Variable" || strSelected === "Both";
			var myEventListenerV = myMenuActionV.eventListeners.add("onInvoke", myVariableEventHandler, false);
			
			var mySeparatorMenuItem = app.menus.item("$ID/RtMouseLayout").menuSeparators.add(LocationOptions.atEnd);
			//var myHeaderMenuItem = app.menus.item("$ID/RtMouseLayout").menuItems.add(app.scriptMenuActions.item("EFI SmartCanvas"));
			var mySelectSameMenuItemS = app.menus.item("$ID/RtMouseLayout").menuItems.add(app.scriptMenuActions.item("Static " + strWhat));
			var mySelectSameMenuItemV = app.menus.item("$ID/RtMouseLayout").menuItems.add(app.scriptMenuActions.item("Variable " + strWhat));
			
		} catch(error) {
			//alert(error);
		}
	}
	
	
	function RemoveAllContextMenu() {
		if(myCheckForMenuItem(myLayoutContextMenu, "Static Text") == true){
			myLayoutContextMenu.menuItems.item("Static Text").remove();
		}
		if(myCheckForMenuItem(myLayoutContextMenu, "Variable Text") == true){
			myLayoutContextMenu.menuItems.item("Variable Text").remove();
		}
		  if(myCheckForMenuItem(myLayoutContextMenu, "Static Image") == true){
			myLayoutContextMenu.menuItems.item("Static Image").remove();
		}
		  if(myCheckForMenuItem(myLayoutContextMenu, "Variable Image") == true){
			myLayoutContextMenu.menuItems.item("Variable Image").remove();
		}
	}
	
	function myCheckForMenuItem(myMenu, myString){
		var myResult = false;
		try{
			var myMenuItem = myMenu.menuItems.item(myString);
			myMenuItem.name;
			myResult = true
		}
		catch(myError){}
		//alert("Menu item found? " + myResult);
		return myResult;
	}

	function myStaticEventHandler(myEvent){
		// Static Event Handler
		if(app.selection.length >= 1){
			
			for(var i=0; i<app.selection.length; i++){
				var myObjectList = new Array;
				switch(app.selection[i].constructor.name){
					case "TextFrame":
						SaveSelectionStatus(app.selection[i].id, "Static", "Text");
						break;
					case "GraphicLine":
					case "Rectangle":
						SaveSelectionStatus(app.selection[i].id, "Static", "Image");
						break;
					case "Oval":
					case "Polygon":
				}
			}
		}
	}

	function myVariableEventHandler(myEvent){
		// Variable Event Handler
		if(app.selection.length >= 1){
			for(var i=0; i<app.selection.length; i++){
				var myObjectList = new Array;
				switch(app.selection[i].constructor.name){
					case "TextFrame":
						SaveSelectionStatus(app.selection[i].id, "Variable", "Text");
						break;
					case "GraphicLine":
					case "Rectangle":
						SaveSelectionStatus(app.selection[i].id, "Variable", "Image");
						break;
					case "Oval":
					case "Polygon":
				}
			}
		}   
	}

	function GetSelectionStatus(selId) {
		try {
			var currentDocument = getCurrentDocument();
			var boxObject = "";
			var blnVariable = false;
			var blnFound = false;
			var blnReturn = "Static";
			var strFieldValue = "";
			
			if(currentDocument.extractLabel(labelName) != "") {
				boxObject = JSON.parse(currentDocument.extractLabel(labelName));
				for(var i=0; i < boxObject.boxProperties.length; i++) {
					if(boxObject.boxProperties[i].id == selId) {
						blnFound = true;
						strFieldValue = boxObject.boxProperties[i].boxVariable.fieldValue;
						if(strFieldValue==false) {
							blnReturn = "Static";
						} else {
							blnReturn = "Variable";
						}
					}
				}
			}
			
		} catch (error) {
			//alert(error);
		}
		
		return blnReturn;
	}
	
	function SaveSelectionStatus(selId, strStatus, strType) {
		var currentDocument = getCurrentDocument();
		var boxObject = "";
		var blnFound=false;
		var strFieldValue = "";
		
		if(currentDocument.extractLabel(labelName) != "") {
			boxObject = JSON.parse(currentDocument.extractLabel(labelName));
			for(var i=0; i < boxObject.boxProperties.length; i++) {
				if(boxObject.boxProperties[i].id == selId) {
					blnFound=true;
					if(strStatus=="Static") {
						boxObject.boxProperties[i].boxVariable.fieldValue = false;
					} else {
						boxObject.boxProperties[i].boxVariable.fieldValue = true;
					}
				}
			}
			//Write the label back
			if(blnFound==false) boxObject.boxProperties.push(CreateSelection(selId, strStatus, strType));
			currentDocument.insertLabel(labelName, JSON.stringify(boxObject));
		} else {
			
			boxObject = {
				boxProperties: [],
				templateProperties: CreateTemplateProperties()
			};
			boxObject.boxProperties.push(CreateSelection(selId, strStatus, strType));
			
			currentDocument.insertLabel(labelName, JSON.stringify(boxObject));
		}
	}
	
	function CreateSelection(selId, strValue, strType) {
		var obj = new Object();
		var objBoxType = new Object();
		var objBoxVariable = new Object();
		
		
		objBoxType.id = "boxType";
		objBoxType.fieldValue = strType + " Object";
		objBoxType.fieldType = "dropdown";
		objBoxType.userCaption = "Type";
		objBoxType.options = ["Text Object", "Image Object"];
		objBoxType.required = false;
		
		objBoxVariable.id = "boxVariable";
		if (strValue=="Static") {
			objBoxVariable.fieldValue = false;
		} else {
			objBoxVariable.fieldValue = true;
		}
		objBoxVariable.fieldType = "checkbox";
		objBoxVariable.userCaption = "Keep frame editable in SmartCanvas";
		objBoxVariable.options = [];
		objBoxVariable.required = false;
		
		obj.id = selId;
		obj.boxType = objBoxType;
		obj.boxVariable = objBoxVariable;
		
		return obj;
	}
	
	function CreateTemplateProperties() {
		var obj = new Object();
		
		obj = {
			templateName: CreateTemplateName(),
			documentWidth: CreateDocumentWidth(),
			documentHeight: CreateDocumentHeight()
		};
		
		return obj;
	}
	
	function CreateTemplateName() {
		var obj = new Object();

		obj.id = "templateName";
		obj.fieldValue = "InDesign2DirectSmile";
		obj.fieldType = "textbox";
		obj.userCaption = "Document Name";
		obj.options = [];
		obj.required = true;
		
		return obj;
	}

	function CreateDocumentWidth() {
		var obj = new Object();
		
		obj.id="documentWidth";
		obj.fieldValue="";
		obj.fieldType="readonly";
		obj.userCaption="Document Width";
		obj.options=[];
		obj.required = false;
			
		return obj;
	}
	
	function CreateDocumentHeight() {
		var obj = new Object();
		
		obj.id="documentHeight";
		obj.fieldValue="";
		obj.fieldType="readonly";
		obj.userCaption="Document Height";
		obj.options=[];
		obj.required = false;
			
		return obj;
	}
	
	function getCurrentDocument() {
		if (app.documents.length > 0) {
			return app.activeDocument;
		} else {
			return null;
		}
	}
	
	/*                      DEBUG FUNCTIONS                */
	
	function getObjectProperties(f) {
		var props = f.reflect.properties;
		var array = [];

		for (var i = 0; i < props.length; i++)
			if(props[i].name){
				try {array.push (props[i].name + ": " + f[props[i].name])} catch (_){}
				array.sort ();
			}

		return array;
	}
	
	function getParameters(object) {
		var aryProperties = getObjectProperties(object);
        var strProperties = "";

        for(var i = 0; i < aryProperties.length; i++) {
            strProperties += aryProperties[i] + '\n';
        }
        Debug(strProperties);
	}
	
	function Debug(text) {
		file = new File("~/Desktop/Menu actions.txt");
		file.encoding = "UTF-8";
		file.open("w");
		file.writeln(text); 
		file.close();
	}


}

