﻿/************************************************************************/
/*  Function: EFISmartCanvasExporter									*/
/*  Purpose: Add Smart Canvas Exporter to Indesign Menu 				*/
/*  Author: Abhinav Kushalnagar											*/
/*  Created Date: 06 December 2016										*/
/*  Version: 1.0														*/
/*																		*/
/*      Installation:                                                   */
/*                                                                      */
/*      1) Place the current file into Scripts/Startup Scripts/         */
/*         (if the folder Startup Scripts doesn't exist, create it)     */
/*                                                                      */
/*                                                                      */
/************************************************************************/
// MenuLoader  
// (#targetengine is not required)  

(function ()
// -------------------------------------  
// Install and/or update the menu/submenu and connect  
// the corresponding menu actions if script files are available  
{
    // Settings and constants  
    // ---  
    try {
        var myMenuItem = app.menus.item("EFI Smart Canvas");
        app.menus.item("$ID/Main").submenus.item("EFI Smart Canvas").remove();
    }
    catch (e) {
        //EFI.Utils.addDebugLog("EFISmartCanvasMenu: " + e);
    }

    //app.menus.item("$ID/Main").submenus.item("EFI Smart Canvas").remove();
    var MENU_NAME = "EFI Smart Canvas",
        FEATURES = [
            { caption: "Export", fileName: "_App.jsx", subName: "" },
            { separator: true, subName: "" },
            { caption: "About", fileName: "_About.jsx", subName: "" }
        ],
        LO_END = LocationOptions.atEnd,
        INDESIGN_ROOT_MENU = app.menus.item('$ID/Main'),
        FEATURE_LOCATION_PATH = (function () {
            var f;
            try { f = app.activeScript; }
            catch (_) { f = File(_.fileName); }
            return f.parent.parent.parent + '/Scripts Panel/EFI/';
        }),
        FEATURE_LOCATION_PATH_ORG = (function () {
            var f;
            try { f = app.activeScript; }
            catch (_) { f = File(_.fileName); }
            return f.parent.parent + '/';
        })();

    // (Re)set the actions  
    // Note: checks also whether script files are available  
    // ---  
    var t, f,
        i = FEATURES.length;
    while (i--) {
        t = FEATURES[i];
        if (t.separator) continue;

        if ((f = File(FEATURE_LOCATION_PATH() + t.fileName)).exists) {
            // The script file exists => create the corresponding action  
            // and directly attach the event listener to the file  
            // (no need to use app.doScript(...) here)  
            // ---  
            (t.action = app.scriptMenuActions.add(t.caption)).
                addEventListener('onInvoke', f);
        }
        else {
            // The script file does not exist => remove that feature  
            // ---  
            alert("Could not find the EFI SmartCanvas plugin at: " + FEATURE_LOCATION_PATH() + t.fileName);
            FEATURES.splice(i, 1);
        }
    }

    // ---  
    // Create/reset the custom menu container *if necessary*  
    // Note:  menus/submenus are application-persistent  
    // ---  
    var mnu = INDESIGN_ROOT_MENU.submenus.itemByName(MENU_NAME);
    if (!mnu.isValid) {
        // Our custom menu hasn't been created yet  
        // ---  
        if (!FEATURES.length) return;
        mnu = INDESIGN_ROOT_MENU.submenus.add(
            MENU_NAME,
            LocationOptions.after,
            INDESIGN_ROOT_MENU.submenus.item('$ID/&Window')
        );
    }
    else {
        // Our custom menu already exists, but we must clear  
        // any sub element in order to rebuild a fresh structure  
        // ---  
        mnu.menuElements.everyItem().remove();

        // If FEATURES is empty, remove the menu itself  
        // ---  
        if (!FEATURES.length) { mnu.remove(); return; }
    }

    // ---  
    // Now, let's fill mnu with respect to FEATURES' order  
    // (Possible submenus are specified in .subName and created on the fly)  
    // ---  
    var s,
        n = FEATURES.length,
        subs = {},
        sub = null;
    for (i = 0; i < n; ++i) {
        t = FEATURES[i];

        // Target the desired submenu  
        // ---  
        sub = (s = t.subName) ?
            (subs[s] || (subs[s] = mnu.submenus.add(s, LO_END))) :
            mnu;

        // Connect the related action OR create a separator  
        // ---  
        if (t.separator)
            sub.menuSeparators.add(LO_END);
        else
            sub.menuItems.add(t.action, LO_END);
    }
})();

